@@ -1,4 +1,4 @@
-# Copyright (C) 2016-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2016-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -26,36 +26,42 @@ import glob
 import os
 import re
 
-from prewikka import siteconfig, utils
+from prewikka import error, siteconfig
 
 
-class ParseError(Exception):
+_sentinel = object()
+
+
+class ConfigError(error.PrewikkaUserError):
+    def __init__(self, message):
+        error.PrewikkaUserError.__init__(self, N_("Configuration error"), message)
+
+
+class ConfigParseError(ConfigError):
     def __init__(self, filename, lineno, line):
-        self._message = _("parse error in \"%(txt)s\" at %(file)s line %(line)d") % {'txt': line.rstrip(), 'file': filename, 'line': lineno}
-
-    def __str__(self):
-        return self._message
+        ConfigError.__init__(self, N_("Parse error in \"%(txt)s\" at %(file)s line %(line)d",
+                                      {'txt': line.rstrip(), 'file': filename, 'line': lineno}))
 
 
-class ConfigParserOption(text_type):
-    def __new__(cls, *args, **kw):
-        return text_type.__new__(cls, args[1])
-
-    def __init__(self, name, value):
-        self.name = name
-        self.value = value
-
-    def __repr__(self):
-        return "<ConfigParserOption %s%s%s>" % (self.name, "=" if self.value else "", self.value or "")
+class ConfigValueError(ConfigError):
+    def __init__(self, value, key):
+        ConfigError.__init__(self, N_("Invalid value '%(value)s' for parameter '%(name)s'",
+                                      {'value': value, 'name': key}))
 
 
-class ConfigParserSection(collections.Mapping):
+class ConfigMissingError(ConfigError, AttributeError):
+    def __init__(self, key):
+        ConfigError.__init__(self, N_("Missing value for parameter '%(name)s'",
+                                      {'name': key}))
+
+
+class ConfigSection(collections.Mapping):
     def __init__(self, name):
         object.__setattr__(self, "_instance_name", name)
         object.__setattr__(self, "_od", collections.OrderedDict())
 
     def __repr__(self):
-        return "ConfigParserSection<%s,%s>" % (self._instance_name, self._od.items())
+        return "ConfigSection<%s,%s>" % (self._instance_name, self._od.items())
 
     def __len__(self):
         return self._od.__len__()
@@ -72,7 +78,7 @@ class ConfigParserSection(collections.Mapping):
     def __getattr__(self, key):
         ret = self._od.get(key, None)
         if ret is None:
-            raise AttributeError
+            raise ConfigMissingError(key)
 
         return ret
 
@@ -88,6 +94,59 @@ class ConfigParserSection(collections.Mapping):
     def get(self, name, default=None):
         return self._od.get(name, default)
 
+    def get_int(self, name, default=0):
+        assert isinstance(default, int)
+
+        value = self.get(name)
+        if value is None:
+            return default
+
+        try:
+            return int(value)
+        except (TypeError, ValueError):
+            raise ConfigValueError(value, name)
+
+    def get_float(self, name, default=0.):
+        assert isinstance(default, float)
+
+        value = self.get(name)
+        if value is None:
+            return default
+
+        try:
+            return float(value)
+        except (TypeError, ValueError):
+            raise ConfigValueError(value, name)
+
+    def get_bool(self, name, default=False):
+        assert isinstance(default, bool)
+
+        value = self.get(name, _sentinel)
+        if value is _sentinel:
+            return default
+
+        if value is None or value.lower() in ['true', 'yes']:
+            return True
+        elif value.lower() in ['false', 'no']:
+            return False
+
+        raise ConfigValueError(value, name)
+
+    def get_size(self, name, default=0):
+        assert isinstance(default, int)
+
+        value = self.get(name)
+        if value is None:
+            return default
+
+        match = re.match(r'(\d+(?:\.\d+)?)([kMGT]?)B', value)
+        if not match:
+            raise ConfigValueError(value, name)
+
+        prefixes = ["", "k", "M", "G", "T"]
+
+        return int(float(match.group(1)) * 1024 ** prefixes.index(match.group(2)))
+
     def keys(self):
         return self._od.keys()
 
@@ -112,7 +171,10 @@ class SectionRoot(list):
             return self[0][key]
 
     def __getattr__(self, attr):
-        return getattr(self[0] if self else ConfigParserSection(""), attr)
+        if not self:
+            self.append(ConfigSection(""))
+
+        return getattr(self[0], attr)
 
     def get_instance_by_name(self, name):
         for section in self:
@@ -151,7 +213,7 @@ class MyConfigParser(object):
             if section.get_instance_name() == instance:
                 return section
 
-        self._sections[name].append(ConfigParserSection(instance))
+        self._sections[name].append(ConfigSection(instance))
         return self._sections[name][-1]
 
     def read_string(self, string):
@@ -182,19 +244,19 @@ class MyConfigParser(object):
 
             result = self.OPTION_REGEXP.match(line)
             if not result:
-                raise ParseError(file.name, lineno + 1, line)
+                raise ConfigParseError(file.name, lineno + 1, line)
 
             if cursection is None:
                 continue
 
             name, value = result.group("name").strip(), result.group("value")
-            cursection[name] = ConfigParserOption(name, value.strip() if value else None)
+            cursection[name] = value.strip() if value else None
 
     def get(self, name, default):
         return self._sections.get(name, default)
 
     def __getattr__(self, key):
-        return self._sections.get(key, SectionRoot())
+        return self._sections.setdefault(key, SectionRoot())
 
     def __len__(self):
         return len(self._sections)
@@ -207,9 +269,10 @@ class Config(MyConfigParser):
         conf_filename = filename or siteconfig.conf_dir + "/prewikka.conf"
         self.read(conf_filename)
 
+        self.basedir = os.path.dirname(conf_filename)
         for fpattern in self.include.keys():
             if not os.path.isabs(fpattern):
-                fpattern = os.path.join(os.path.dirname(conf_filename), fpattern)
+                fpattern = os.path.join(self.basedir, fpattern)
 
             # Files are loaded in alphabetical order
             for fname in sorted(glob.glob(fpattern)):
