@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -27,13 +27,12 @@ import operator
 import pkgutil
 import re
 import time
-import types
 from datetime import datetime
 
 import pkg_resources
 import preludedb
 from prewikka import compat, error, log, utils, version
-from prewikka.utils import cache, json
+from prewikka.utils import cache
 
 
 NotNone = object
@@ -51,17 +50,16 @@ class DatabaseSchemaError(DatabaseError):
     name = N_("Database schema error")
 
 
-
 # Internal workaround since SWIG generated exception use class RuntimeError
 def _fix_exception(func):
-        def inner(self, *args, **kwargs):
-            try:
-                ret = func(self, *args, **kwargs)
-            except RuntimeError as e:
-                raise DatabaseError(message=text_type(e))
+    def inner(self, *args, **kwargs):
+        try:
+            ret = func(self, *args, **kwargs)
+        except RuntimeError as e:
+            raise DatabaseError(message=text_type(e))
 
-            return ret
-        return inner
+        return ret
+    return inner
 
 
 def _use_flock(func):
@@ -83,17 +81,19 @@ def _use_flock(func):
 def use_transaction(func):
     @functools.wraps(func)
     def inner(self, *args, **kwargs):
-        if env.db._transaction_state:
+        db = getattr(self, "__db", env.db)
+
+        if db._transaction_state:
             return func(self, *args, **kwargs)
 
-        env.db.transaction_start()
+        db.transaction_start()
         try:
             ret = func(self, *args, **kwargs)
         except:
-            env.db.transaction_abort()
+            db.transaction_abort()
             raise
 
-        env.db.transaction_end()
+        db.transaction_end()
         return ret
 
     return inner
@@ -104,15 +104,17 @@ def use_lock(table):
 
         @use_transaction
         def inner(self, *args, **kwargs):
-            env.db._lock_table(table)
+            db = getattr(self, "__db", env.db)
+
+            db._lock_table(table)
 
             try:
                 ret = func(self, *args, **kwargs)
             except:
-                env.db._unlock_table(table)
+                db._unlock_table(table)
                 raise
 
-            env.db._unlock_table(table)
+            db._unlock_table(table)
             return ret
 
         return inner
@@ -135,7 +137,7 @@ class SQLScript(object):
     """
 
     __metaclass__ = abc.ABCMeta
-    __all__ = [ "type", "branch", "version", "from_branch" ]
+    __all__ = ["type", "branch", "version", "from_branch"]
 
     type = "update"
     branch = None
@@ -147,10 +149,12 @@ class SQLScript(object):
         self.query_logs = []
         self._module_name = dbup._module_name
         self._full_module_name = dbup._full_module_name
-        self._query_filter = { "sqlite3": self._mysql2sqlite, "pgsql": self._mysql2pgsql, "mysql": self._mysqlhandler }[self.db.getType()]
+        self._query_filter = {"sqlite3": self._mysql2sqlite,
+                              "pgsql": self._mysql2pgsql,
+                              "mysql": self._mysqlhandler}[self.db.get_type()]
 
         if self.type in ("install", "update"):
-           if not self.version:
+            if not self.version:
                 raise Exception("SQL %s script require 'version' attribute" % self.type)
 
         elif self.type == "branch":
@@ -165,38 +169,42 @@ class SQLScript(object):
         return input
 
     def _mysql2pgsql(self, input):
-        _stbl = [ ("#.*", ""),
-                  (" INT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT", " SERIAL PRIMARY KEY"),
-                  ("BIGINT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT", "BIGSERIAL PRIMARY KEY"),
-                  ("BLOB", "BYTEA"),
-                  (" TINYINT UNSIGNED ", " INT4 "),
-                  (" TINYINT ", " INT2 "),
-                  (" SMALLINT UNSIGNED ", " INT8 "),
-                  (" SMALLINT ", " INT4 "),
-                  (" BIGINT UNSIGNED ", " INT8 "),
-                  (" BIGINT ", " INT8 "),
-                  (" INT(EGER)? UNSIGNED ", " INT8 "),
-                  (" INT(EGER)? ", " INT4 "),
-                  ("DATETIME", "TIMESTAMP"),
-                  ("ENGINE=InnoDB", ""),
-                  ("\"([^\"]*)\"", "'\\1'"),
-                  ("\"\([^\"]*\)\"", "'\1'"),
-                  ("(\S*) ENUM\((.*)\)", "\\1 TEXT CHECK (\\1 IN (\\2))"),
-                  ("VARCHAR[ ]*[^)]+\)", "TEXT"),
-                  ("(DROP INDEX [^ ]*) ON [^;]*", "\\1")]
+        _stbl = [
+            ("#.*", ""),
+            (" INT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT", " SERIAL PRIMARY KEY"),
+            ("BIGINT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT", "BIGSERIAL PRIMARY KEY"),
+            ("BLOB", "BYTEA"),
+            (" TINYINT UNSIGNED ", " INT4 "),
+            (" TINYINT ", " INT2 "),
+            (" SMALLINT UNSIGNED ", " INT8 "),
+            (" SMALLINT ", " INT4 "),
+            (" BIGINT UNSIGNED ", " INT8 "),
+            (" BIGINT ", " INT8 "),
+            (" INT(EGER)? UNSIGNED ", " INT8 "),
+            (" INT(EGER)? ", " INT4 "),
+            ("DATETIME", "TIMESTAMP"),
+            ("ENGINE=InnoDB", ""),
+            ("\"([^\"]*)\"", "'\\1'"),
+            ("\"\([^\"]*\)\"", "'\1'"),
+            ("(\S*) ENUM\((.*)\)", "\\1 TEXT CHECK (\\1 IN (\\2))"),
+            ("VARCHAR[ ]*[^)]+\)", "TEXT"),
+            ("(DROP INDEX [^ ]*) ON [^;]*", "\\1")
+        ]
 
         return self._sub(_stbl, input)
 
     def _mysql2sqlite(self, input):
-        _stbl = [ ("#.*", ""),
-                  ("[a-zA-Z]*INT ", "INTEGER "),
-                  ("UNSIGNED ", ""),
-                  ("ENUM[ ]*[^)]+\)", "TEXT"),
-                  ("VARCHAR[ ]*[^)]+\)", "TEXT"),
-                  ("AUTO_INCREMENT", "AUTOINCREMENT"),
-                  ("ENGINE=InnoDB", ""),
-                  ("ALTER TABLE [^ ]* DROP.*", ""),
-                  ("(DROP INDEX [^ ]*) ON [^;]*", "\\1")]
+        _stbl = [
+            ("#.*", ""),
+            ("[a-zA-Z]*INT ", "INTEGER "),
+            ("UNSIGNED ", ""),
+            ("ENUM[ ]*[^)]+\)", "TEXT"),
+            ("VARCHAR[ ]*[^)]+\)", "TEXT"),
+            ("AUTO_INCREMENT", "AUTOINCREMENT"),
+            ("ENGINE=InnoDB", ""),
+            ("ALTER TABLE [^ ]* DROP.*", ""),
+            ("(DROP INDEX [^ ]*) ON [^;]*", "\\1")
+        ]
 
         return self._sub(_stbl, input)
 
@@ -207,7 +215,7 @@ class SQLScript(object):
         for q in self._query_filter(input).split(";"):
             q = q.strip()
             if q:
-                self.query_logs.append(q);
+                self.query_logs.append(q)
                 self.db.query(q)
 
     @abc.abstractmethod
@@ -216,15 +224,23 @@ class SQLScript(object):
 
     @use_transaction
     def apply(self):
-        log.getLogger().info("%s: please standby while %s is applied", self._full_module_name, text_type(self))
+        log.get_logger().info("%s: please standby while %s is applied", self._full_module_name, text_type(self))
 
         self.run()
 
         if self.type == "install":
-            env.db.upsert("Prewikka_Module_Registry", ("module", "branch", "version"), ((self._full_module_name, self.branch, self.version),), pkey=("module",))
+            self.db.upsert(
+                "Prewikka_Module_Registry",
+                ("module", "branch", "version"),
+                ((self._full_module_name, self.branch, self.version),),
+                pkey=("module",)
+            )
 
         elif self.type == "update":
-            self.db.query("UPDATE Prewikka_Module_Registry SET version=%s WHERE module=%s%s" % (self.db.escape(self.version), self.db.escape(self._full_module_name), self.db._chknull("branch", self.branch)))
+            self.db.query("UPDATE Prewikka_Module_Registry SET version=%s WHERE module=%s%s" %
+                          (self.db.escape(self.version),
+                           self.db.escape(self._full_module_name),
+                           self.db._chknull("branch", self.branch)))
 
         elif self.type == "branch":
             self.db.query("UPDATE Prewikka_Module_Registry SET branch=%s, version=%s, enabled=1 WHERE module=%s",
@@ -250,28 +266,24 @@ class DatabaseHelper(object):
         return self.__dict__.get(x, getattr(env.db, x))
 
 
-
 class DatabaseUpdateHelper(DatabaseHelper):
-    _default_modinfo = ModuleInfo(None, None, True)
-
     def _init_version_attr(self):
         if self._initialized:
             return
 
-        module = self.modinfos.get(self._full_module_name, self._default_modinfo)
+        module = env.db.modinfos.get(self._full_module_name, self._default_modinfo)
 
         self._from_branch = module.branch
         self._from_version = module.version
         self._need_enable = not(module.enabled)
         self._initialized = True
 
-    def __init__(self, module_name, reqversion, reqbranch=None):
-        DatabaseHelper.__init__(self) #for use_transaction
-
+    def __init__(self, module_name, reqversion, reqbranch=None, enabled=True):
         self._reqbranch = reqbranch
         self._reqversion = reqversion
         self._module_name = module_name.split(":")[0]
         self._full_module_name = module_name
+        self._default_modinfo = ModuleInfo(None, None, enabled)
         self._initialized = False
 
     def check(self):
@@ -301,7 +313,7 @@ class DatabaseUpdateHelper(DatabaseHelper):
             try:
                 yield i.load().__path__[0]
             except Exception as e:
-                log.getLogger().exception("[%s]: error loading SQL updates: %s", self._full_module_name, e)
+                log.get_logger().exception("[%s]: error loading SQL updates: %s", self._full_module_name, e)
 
     def _get_schema_list(self, **kwargs):
         from_version = to_version = None
@@ -318,7 +330,8 @@ class DatabaseUpdateHelper(DatabaseHelper):
             try:
                 mod = importer.find_module(package_name).load_module(package_name).SQLUpdate(self)
             except Exception as e:
-                log.getLogger().exception("[%s]: error loading SQL update '%s' : %s" % (self._full_module_name, package_name, e))
+                log.get_logger().exception("[%s]: error loading SQL update '%s' : %s" %
+                                          (self._full_module_name, package_name, e))
                 continue
 
             if any(kwargs[k] != getattr(mod, k) for k in kwargs.keys()):
@@ -326,17 +339,15 @@ class DatabaseUpdateHelper(DatabaseHelper):
 
             version = pkg_resources.parse_version(mod.version)
             if (not from_version or (version > from_version)) and (not to_version or (version <= to_version)):
-                    yield mod
-
+                yield mod
 
     def _resolve_branch_switch(self, curbranch, curversion, outstack=[]):
-
         for upd in self._list(from_branch=(curbranch, curversion), type="branch"):
             if upd.branch == self._reqbranch and pkg_resources.parse_version(upd.version) <= pkg_resources.parse_version(self._reqversion):
                 return outstack + [upd]
 
             elif upd in outstack:
-                log.getLogger().warning("cyclic branch dependencies detected: %s",  " -> ".join(text_type(i) for i in outstack + [upd]))
+                log.get_logger().warning("cyclic branch dependencies detected: %s",  " -> ".join(text_type(i) for i in outstack + [upd]))
                 continue
 
             else:
@@ -344,6 +355,11 @@ class DatabaseUpdateHelper(DatabaseHelper):
                 if ret:
                     return ret
 
+        for upd in self._list(from_version=curversion, branch=curbranch, type="update"):
+            ret = self._resolve_branch_switch(upd.branch, upd.version, outstack=outstack[:] + [upd])
+            if ret:
+                return ret
+
         return []
 
     def _list(self, *args, **kwargs):
@@ -387,7 +403,7 @@ class DatabaseUpdateHelper(DatabaseHelper):
         from_version, prev = self._from_version, []
 
         if not from_version:
-            prev = [ self._get_install_schema() ]
+            prev = [self._get_install_schema()]
 
         elif self._from_branch != self._reqbranch:
             prev = self._get_branch_update()
@@ -412,7 +428,7 @@ class DatabaseUpdateHelper(DatabaseHelper):
 
     @use_transaction
     def _apply(self):
-        [ update.apply() for update in self.list() ]
+        [update.apply() for update in self.list()]
         self.check()
 
     @_use_flock
@@ -428,33 +444,29 @@ class DatabaseUpdateHelper(DatabaseHelper):
         return self._from_version
 
 
-class DatabaseCommon(preludedb.SQL):
+class DatabaseCommon(object):
     required_branch = version.__branch__
-    required_version = "2"
+    required_version = "0"
 
     NotNone = NotNone
     __sentinel = object()
-    __ALL_PROPERTIES = object()
 
-    __TRANSACTION_STATE_NONE  = 0
+    __TRANSACTION_STATE_NONE = 0
     __TRANSACTION_STATE_BEGIN = 1
     __TRANSACTION_STATE_QUERY = 2
 
-    @cache.memoize_property("modinfos_cache")
-    def modinfos(self):
-        try:
-            rows = self.query("SELECT module, branch, version, enabled FROM Prewikka_Module_Registry")
-        except:
-            return {}
-
-        return dict((i[0], ModuleInfo(i[1], i[2], int(i[3]))) for i in rows)
+    def _get_prefilter(self, v):
+        if not(isinstance(v, (text_type, bytes))) and isinstance(v, collections.Iterable):
+            return self.__ESCAPE_PREFILTER["iterable"]
+        else:
+            return self.__ESCAPE_PREFILTER.get(type(v))
 
     def _prefilter_iterate(self, l):
         tmp = []
         for v in l:
             tmp.append(text_type(self.escape(v)))
 
-        if self.__ESCAPE_PREFILTER.get(type(v)) == self._prefilter_iterate:
+        if self._get_prefilter(v) == self._prefilter_iterate:
             fmt = '%s'
         else:
             fmt = '(%s)'
@@ -462,34 +474,22 @@ class DatabaseCommon(preludedb.SQL):
         return fmt % ', '.join(tmp)
 
     @_fix_exception
-    def __init__(self, config):
-        env.db = self
-
+    def __init__(self, settings):
         self.__ESCAPE_PREFILTER = {
-                datetime: lambda dt: self.escape(self.datetime(dt)),
-                set: self._prefilter_iterate,
-                list: self._prefilter_iterate,
-                tuple: self._prefilter_iterate,
-                types.GeneratorType: self._prefilter_iterate
+            bool: int,
+            datetime: lambda dt: self.escape(self.datetime(dt)),
+            "iterable": self._prefilter_iterate,
         }
 
         self._transaction_state = self.__TRANSACTION_STATE_NONE
 
-        settings = { "host": "localhost", "name": "prewikka", "user": "prewikka", "type": "mysql" }
-        stpl = tuple((k, v) for k, v in config.items())
-        settings.update(stpl)
+        stpl = tuple((k, v) for k, v in settings.items())
+        self._db = preludedb.SQL(settings)
 
-        preludedb.SQL.__init__(self, settings)
-
-        self._version = self.getServerVersion()
+        self._version = self._db.getServerVersion()
         self._dbhash = hash(stpl)
         self._dbtype = settings["type"]
 
-        dh = DatabaseUpdateHelper("prewikka", self.required_version, self.required_branch)
-        dh.apply()
-
-        self._last_plugin_activation_change = self._get_last_plugin_changed()
-
     @staticmethod
     def parse_datetime(date):
         if "." in date:
@@ -536,7 +536,7 @@ class DatabaseCommon(preludedb.SQL):
 
     def query(self, sql, *args, **kwargs):
         if self._transaction_state == self.__TRANSACTION_STATE_BEGIN:
-            self.transactionStart()
+            self._db.transactionStart()
             self._transaction_state = self.__TRANSACTION_STATE_QUERY
 
         if args:
@@ -544,10 +544,10 @@ class DatabaseCommon(preludedb.SQL):
         elif kwargs:
             sql = sql % dict((key, self.escape(value)) for key, value in kwargs.items())
 
-        return preludedb.SQL.query(self, sql)
+        return self._db.query(sql)
 
     def _chk(self, key, value, join="AND"):
-        if value is not None and value is not self.__ALL_PROPERTIES:
+        if value is not None:
             return " %s %s = %s" % (join, key, self.escape(value))
 
         return ""
@@ -565,43 +565,27 @@ class DatabaseCommon(preludedb.SQL):
         else:
             return (value,)
 
-    def getType(self):
+    def get_type(self):
         return self._dbtype
 
     def escape(self, data):
-        prefilter = self.__ESCAPE_PREFILTER.get(type(data))
+        prefilter = self._get_prefilter(data)
         if prefilter:
             return prefilter(data)
 
         if not isinstance(data, compat.STRING_TYPES):
             return data if data is not None else "NULL"
 
-        return preludedb.SQL.escape(self, data)
+        return self._db.escape(data)
 
-    def is_plugin_active(self, plugin):
-        plugin = self.modinfos.get(plugin)
-        if plugin:
-            return plugin.enabled == 1
+    def escape_binary(self, data):
+        return self._db.escapeBinary(data)
 
-        return True
+    def unescape_binary(self, data):
+        return self._db.unescapeBinary(data)
 
-    def _get_last_plugin_changed(self):
-        rows = self.query("SELECT time FROM Prewikka_Module_Changed")[0][0]
-        return utils.timeutil.get_timestamp_from_string(rows)
-
-    def has_plugin_changed(self):
-        last = self._get_last_plugin_changed()
-
-        if last <= self._last_plugin_activation_change:
-            return False
-
-        self._last_plugin_activation_change = last
-        self.modinfos_cache.clear()
-
-        return True
-
-    def trigger_plugin_change(self):
-        self.query("UPDATE Prewikka_Module_Changed SET time=current_timestamp")
+    def get_last_insert_ident(self):
+        return self._db.getLastInsertIdent()
 
     def _get_merge_value(self, merged, field, rownum):
         value = merged[field]
@@ -657,13 +641,13 @@ class DatabaseCommon(preludedb.SQL):
 
     def transaction_end(self):
         if self._transaction_state == self.__TRANSACTION_STATE_QUERY:
-            self.transactionEnd()
+            self._db.transactionEnd()
 
         self._transaction_state = self.__TRANSACTION_STATE_NONE
 
     def transaction_abort(self):
         if self._transaction_state == self.__TRANSACTION_STATE_QUERY:
-            self.transactionAbort()
+            self._db.transactionAbort()
 
         self._transaction_state = self.__TRANSACTION_STATE_NONE
 
@@ -693,8 +677,8 @@ class MySQLDatabase(DatabaseCommon):
         if retfmt:
             wh = []
             for row in values_rows:
-                vl = " AND ".join([ "%s = %s" % (field, self.escape(row[i])) for i, field in enumerate(pkey) ])
-                wh.append("(%s)" % (vl))
+                vl = " AND ".join(["%s = %s" % (field, self.escape(row[i])) for i, field in enumerate(pkey)])
+                wh.append("(%s)" % vl)
 
             return self.query("SELECT %s FROM %s WHERE %s" % (retfmt, table, " OR ".join(wh)))
 
@@ -707,9 +691,6 @@ class MySQLDatabase(DatabaseCommon):
 
 
 class PgSQLDatabase(DatabaseCommon):
-    def __init__(self, *args, **kwargs):
-        DatabaseCommon.__init__(self, *args, **kwargs)
-
     def _lock_table(self, table):
         self.query("LOCK TABLE %s IN EXCLUSIVE MODE" % ", ".join(self._mklist(table)))
 
@@ -718,7 +699,7 @@ class PgSQLDatabase(DatabaseCommon):
         out = {}
         typemap = {"bigint": "integer", "smallint": "integer", "character varying": "text"}
 
-        for field, _type, defval in env.db.query("SELECT column_name, data_type, column_default FROM information_schema.columns WHERE table_name = %s", table.lower()):
+        for field, _type, defval in self.query("SELECT column_name, data_type, column_default FROM information_schema.columns WHERE table_name = %s", table.lower()):
             out[field] = utils.AttrObj(type=_type, generic_type=typemap.get(_type, _type), default=defval, auto_increment="nextval" in (defval or ""))
 
         return out
@@ -856,21 +837,95 @@ class PgSQLDatabase(DatabaseCommon):
 
 
 class NoDatabase(DatabaseCommon):
-    def __init__(self, *args, **kwargs):
-        DatabaseCommon.__init__(self, *args, **kwargs)
-
     def query(self, *args, **kwargs):
         raise error.PrewikkaUserError(N_("Database configuration error"), N_("Only MySQL and PostgreSQL databases are supported at the moment"))
 
 
 class Database(object):
-    def __new__(cls, config):
-        type = config.get("type")
+    def __new__(cls, settings):
+        type = settings.get("type")
         if type == "pgsql":
-            return PgSQLDatabase(config)
+            return PgSQLDatabase(settings)
 
         elif type == "mysql":
-            return MySQLDatabase(config)
+            return MySQLDatabase(settings)
 
         else:
-            return NoDatabase(config)
+            return NoDatabase(settings)
+
+
+class PrewikkaDatabase(object):
+    def __new__(cls, config):
+        settings = {"host": "localhost", "name": "prewikka", "user": "prewikka", "type": "mysql"}
+        settings.update(config.items())
+
+        type = settings["type"]
+        if type == "pgsql":
+            return PrewikkaPgSQLDatabase(settings)
+
+        elif type == "mysql":
+            return PrewikkaMySQLDatabase(settings)
+
+        else:
+            return PrewikkaNoDatabase(settings)
+
+
+class PrewikkaDatabaseCommon(DatabaseCommon):
+    def __init__(self):
+        env.db = self
+        dh = DatabaseUpdateHelper("prewikka", self.required_version, self.required_branch)
+        dh.apply()
+
+        self._last_plugin_activation_change = self._get_last_plugin_changed()
+
+    @cache.memoize_property("modinfos_cache")
+    def modinfos(self):
+        try:
+            rows = self.query("SELECT module, branch, version, enabled FROM Prewikka_Module_Registry")
+        except:
+            return {}
+
+        return dict((i[0], ModuleInfo(i[1], i[2], int(i[3]))) for i in rows)
+
+    def is_plugin_active(self, plugin):
+        module = self.modinfos.get(plugin.full_module_name)
+        if module:
+            return module.enabled == 1
+
+        return plugin.plugin_enabled
+
+    def _get_last_plugin_changed(self):
+        rows = self.query("SELECT time FROM Prewikka_Module_Changed")[0][0]
+        return utils.timeutil.get_timestamp_from_string(rows)
+
+    def has_plugin_changed(self):
+        last = self._get_last_plugin_changed()
+
+        if last <= self._last_plugin_activation_change:
+            return False
+
+        self._last_plugin_activation_change = last
+        self.modinfos_cache.clear()
+
+        return True
+
+    def trigger_plugin_change(self):
+        self.query("UPDATE Prewikka_Module_Changed SET time=current_timestamp")
+
+
+class PrewikkaPgSQLDatabase(PgSQLDatabase, PrewikkaDatabaseCommon):
+    def __init__(self, settings):
+        PgSQLDatabase.__init__(self, settings)
+        PrewikkaDatabaseCommon.__init__(self)
+
+
+class PrewikkaMySQLDatabase(MySQLDatabase, PrewikkaDatabaseCommon):
+    def __init__(self, settings):
+        MySQLDatabase.__init__(self, settings)
+        PrewikkaDatabaseCommon.__init__(self)
+
+
+class PrewikkaNoDatabase(NoDatabase, PrewikkaDatabaseCommon):
+    def __init__(self, settings):
+        NoDatabase.__init__(self, settings)
+        PrewikkaDatabaseCommon.__init__(self)
