@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -48,12 +48,13 @@ class PrewikkaError(PrewikkaException):
     name = N_("An unexpected condition happened")
     message = ""
     details = ""
+    output = ""
     code = 500
     log_priority = log.ERROR
     display_traceback = True
     errno = None
 
-    def __init__(self, message, name=None, details=None, log_priority=None, log_user=None, template=None, code=None):
+    def __init__(self, message, name=None, details=None, log_priority=None, log_user=None, template=None, code=None, output=None):
         if name is not None:
             self.name = name
 
@@ -72,13 +73,16 @@ class PrewikkaError(PrewikkaException):
         if log_priority:
             self.log_priority = log_priority
 
+        if output:
+            self.output = output
+
         self.traceback = self._get_traceback()
         self.log_user = log_user
 
     def _setup_template(self, template, ajax_error):
         dataset = template.dataset()
 
-        for i in ("name", "message", "details", "code", "traceback", "errno"):
+        for i in ("name", "message", "details", "code", "traceback", "errno", "output"):
             dataset[i] = getattr(self, i)
 
         dataset["is_ajax_error"] = ajax_error
@@ -87,11 +91,10 @@ class PrewikkaError(PrewikkaException):
         return dataset
 
     def _html_respond(self):
-        from prewikka import baseview
-        return baseview.BaseView().respond(self._setup_template(self.template, False), self.code)
+        return env.viewmanager.get_baseview().respond(self._setup_template(self.template, False), self.code)
 
     def _get_traceback(self):
-        if self.display_traceback and env.config.general.get("enable_error_traceback") not in ('no', 'false'):
+        if self.display_traceback and env.config.general.get_bool("enable_error_traceback", True):
             return sys.exc_info()
 
     def respond(self):
@@ -112,14 +115,15 @@ class PrewikkaError(PrewikkaException):
         if details:
             return "%s: %s" % (message, details)
 
-        return message
+        # message might be an exception class
+        return text_type(message)
 
     def __str__(self):
         return self._format_error(self.message, self.details)
 
     def __json__(self):
         dset = self._setup_template(PrewikkaError.template, True)
-        return { "content": dset.render() }
+        return {"content": dset.render(), "error": True}
 
 
 class PrewikkaUserError(PrewikkaError):
