@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -36,17 +36,12 @@ class Log(object):
     def __init__(self, conf):
         self._logger = None
 
-        for logconfig in getattr(conf, "log", ()):
-
-            config = { }
-            for key, value in logconfig.items():
-                config[key] = text_type(value)
-
+        for instance in conf:
             self._logger = logging.getLogger()
             self._logger.setLevel(logging.NOTSET)
-            self._logger.addHandler(self._getHandler(config, logconfig.get_instance_name()))
+            self._logger.addHandler(self._get_handler(instance))
 
-    def _getSyslogHandlerAddress(self):
+    def _get_syslog_handler_address(self):
         for f in ("/dev/log", "/var/run/log", "/var/run/syslog"):
             try:
                 if stat.S_ISSOCK(os.stat(f).st_mode):
@@ -54,30 +49,29 @@ class Log(object):
             except:
                 pass
 
-        return ("localhost", 514)
+        return "localhost", 514
 
-
-    def _getHandler(self, config, logtype='syslog'):
-        logtype = logtype.lower()
+    def _get_handler(self, config):
+        logtype = (config.get_instance_name() or "syslog").lower()
         level = config.get("level", "")
 
         if logtype == 'file':
-            hdlr = logging.FileHandler(config["file"])
+            hdlr = logging.FileHandler(config.file)
 
         elif logtype == 'nteventlog':
             hdlr = logging.handlers.NTEventLogHandler("Prewikka", logtype='Application')
 
-        elif logtype in ['syslog', 'unix']:
-            hdlr = logging.handlers.SysLogHandler(self._getSyslogHandlerAddress(), facility=logging.handlers.SysLogHandler.LOG_DAEMON)
+        elif logtype in ('syslog', 'unix'):
+            hdlr = logging.handlers.SysLogHandler(self._get_syslog_handler_address(), facility=logging.handlers.SysLogHandler.LOG_DAEMON)
 
-        elif logtype in ['smtp']:
-            hdlr = logging.handlers.SMTPHandler(config["host"], config["from"], config["to"].split(", "), config["subject"])
+        elif logtype == 'smtp':
+            hdlr = logging.handlers.SMTPHandler(config.host, getattr(config, "from"), config.to.split(", "), config.subject)
 
-        elif logtype in ['stderr']:
+        elif logtype == 'stderr':
             hdlr = logging.StreamHandler(sys.stderr)
 
         else:
-            raise ValueError(_("Unknown logtype specified: '%s'") % logtype)
+            raise ValueError("Unknown logtype specified: '%s'" % logtype)
 
         format = 'prewikka (pid:%(process)d) %(name)s %(levelname)s: %(message)s'
         if logtype in ['file', 'stderr']:
@@ -104,7 +98,6 @@ class Log(object):
 
         return hdlr
 
-
     def _format_header(self):
         if not env.request.web:
             return ""
@@ -119,16 +112,15 @@ class Log(object):
 
         flags = ""
         if env.request.web.is_xhr:
-           flags = " (xhr)"
+            flags = " (xhr)"
         elif env.request.web.is_stream:
-           flags = " (sse)"
+            flags = " (sse)"
 
         return "".join((hdr, env.request.web.path, flags, "]"))
 
-
     def _get_log(self, details):
         hdr = self._format_header()
-        hdr = [ hdr ] if hdr else []
+        hdr = [hdr] if hdr else []
 
         if isinstance(details, Exception):
             details = " ".join([text_type(getattr(details, "code", 500)), text_type(details)])
@@ -156,11 +148,14 @@ class Log(object):
             self._logger.critical(self._get_log(message))
 
     def log(self, priority, message):
-        return { DEBUG: self.debug,
-                 INFO: self.info,
-                 WARNING: self.warning,
-                 ERROR: self.error,
-                 CRITICAL: self.critical }[priority](message)
+        return {
+            DEBUG: self.debug,
+            INFO: self.info,
+            WARNING: self.warning,
+            ERROR: self.error,
+            CRITICAL: self.critical
+        }[priority](message)
 
-def getLogger(name=__name__):
-        return logging.getLogger(name)
+
+def get_logger(name=__name__):
+    return logging.getLogger(name)
