@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -19,19 +19,21 @@
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+import errno
+import gc
 import os
+import socket
 
 import pkg_resources
 import prelude
 import preludedb
-from prewikka import (auth, config, database, dataprovider, env, error, hookmanager, idmefdatabase, localization, log,
-                      menu, pluginmanager, renderer, resolve, response, siteconfig, version, view)
-from prewikka.utils import viewhelpers
+from prewikka import (auth, cli, config, database, dataprovider, error, history, hookmanager, link, localization,
+                      log, menu, pluginmanager, renderer, resolve, response, siteconfig, usergroup, version, view)
 
 try:
-    from threading import Lock, local
+    from threading import Lock
 except ImportError:
-    from dummy_threading import Lock, local
+    from dummy_threading import Lock
 
 
 _core_cache = {}
@@ -39,7 +41,7 @@ _core_cache_lock = Lock()
 
 
 class Core(object):
-    def _checkVersion(self):
+    def _check_version(self):
         error_type = _("Version Requirement error")
         if not prelude.checkVersion(siteconfig.libprelude_required_version):
             raise error.PrewikkaUserError(error_type,
@@ -51,9 +53,8 @@ class Core(object):
                                           N_("Prewikka %(vPre)s requires libpreludedb %(vLib)s or higher",
                                              {'vPre': version.__version__, 'vLib': siteconfig.libpreludedb_required_version}))
 
-
     @staticmethod
-    def from_config(path=None, threaded=False):
+    def from_config(path=None, threaded=False, autoupdate=False):
         global _core_cache
         global _core_cache_lock
 
@@ -62,12 +63,13 @@ class Core(object):
 
         with _core_cache_lock:
             if path not in _core_cache:
-                _core_cache[path] = Core(path)
+                _core_cache[path] = Core(path, autoupdate)
 
         return _core_cache[path]
 
-    def __init__(self, filename=None):
-        env.auth = None # In case of database error
+    def __init__(self, filename=None, autoupdate=False):
+        self.autoupdate = autoupdate
+        env.auth = None  # In case of database error
         env.config = config.Config(filename)
 
         env.config.general.setdefault("default_theme", "cs")
@@ -76,19 +78,17 @@ class Core(object):
         env.config.general.setdefault("default_timezone", localization.get_system_timezone())
         env.config.general.reverse_path = env.config.general.get("reverse_path", "").rstrip("/")
 
-        env.log = log.Log(env.config)
+        env.log = log.Log(env.config.log)
         env.log.info("Starting Prewikka")
 
-        env.dns_max_delay = float(env.config.general.get("dns_max_delay", 0))
+        env.dns_max_delay = env.config.general.get_float("dns_max_delay", 0.)
 
-        val = env.config.general.get("external_link_new_window", "true")
-        if val is None or val.lower() in ["true", "yes"]:
+        if env.config.general.get_bool("external_link_new_window", True):
             env.external_link_target = "_blank"
         else:
             env.external_link_target = "_self"
 
-        details = env.config.general.get("enable_details", "false")
-        env.enable_details = details is None or details.lower() in ["true", "yes"]
+        env.enable_details = env.config.general.get_bool("enable_details", False)
 
         env.host_details_url = env.config.general.get("host_details_url", "https://www.prelude-siem.com/host_details.php")
         env.port_details_url = env.config.general.get("port_details_url", "https://www.prelude-siem.com/port_details.php")
@@ -100,46 +100,49 @@ class Core(object):
         env.viewmanager = view.ViewManager()
         env.htdocs_mapping.update((("prewikka", pkg_resources.resource_filename(__name__, 'htdocs')),))
 
-        custom_theme = env.config.interface.get("custom_theme", None)
-        if custom_theme:
-            if os.path.isdir("%s%s" % (os.path.sep, custom_theme)):
-                env.htdocs_mapping.update((("custom", custom_theme),))
-            else:
-                env.htdocs_mapping.update((("custom", pkg_resources.resource_filename(custom_theme, 'htdocs')),))
-
         try:
             self._prewikka_initialized = False
             self._prewikka_init_if_needed()
         except:
             pass
 
+    def _load_custom_theme(self):
+        custom_theme = env.config.interface.get("custom_theme")
+        if custom_theme is None:
+            return
+
+        if os.path.isdir("%s%s" % (os.path.sep, custom_theme)):
+            env.htdocs_mapping.update((("custom", custom_theme),))
+        else:
+            try:
+                env.htdocs_mapping.update((("custom", pkg_resources.resource_filename(custom_theme, 'htdocs')),))
+            except ImportError:
+                raise config.ConfigValueError(custom_theme, "custom_theme")
+
     def _prewikka_init_if_needed(self):
         if self._prewikka_initialized is True:
-            return self._reload_plugin_if_needed()
+            return self.reload_plugin_if_needed()
 
         try:
-            self._checkVersion()
-            env.db = database.Database(env.config.database)
-            env.idmef_db = idmefdatabase.IDMEFDatabase(env.config.idmef_database)
-            self._initURL()
-            self._loadPlugins()
+            self._load_custom_theme()
+            self._check_version()
+            env.db = database.PrewikkaDatabase(env.config.database)
+            history.init()
+            self._load_plugins()
             self._prewikka_initialized = True
         except error.PrewikkaError as e:
             self._prewikka_initialized = e
         except Exception as e:
             self._prewikka_initialized = error.PrewikkaError(e, name=_("Initialization error"))
+        finally:
+            # Needed for Database object
+            gc.collect()
 
         if isinstance(self._prewikka_initialized, Exception):
             env.log.log(self._prewikka_initialized.log_priority, text_type(self._prewikka_initialized))
+            self._unregister_plugin_data()
             raise self._prewikka_initialized
 
-    def _initURL(self):
-        env.url = {}
-        for urlconf in env.config.url:
-            env.url[urlconf.get_instance_name()] = {}
-            for label, url in urlconf.items():
-                env.url[urlconf.get_instance_name()][label] = url
-
     def _load_auth_or_session(self, typename, plugins, name, config=config.SectionRoot()):
         if name not in plugins:
             raise error.PrewikkaUserError(
@@ -152,20 +155,29 @@ class Core(object):
         setattr(env, typename, obj)
         obj.init(config)
 
-    def _loadPlugins(self):
+    def _unregister_plugin_data(self):
+        list(hookmanager.trigger("HOOK_PLUGINS_RELOAD"))
+        hookmanager.unregister(exclude=["HOOK_PLUGINS_RELOAD"])
+        cli.unregister()
+        usergroup.ACTIVE_PERMISSIONS = usergroup.Permissions()
+
+    def _load_plugins(self):
+        env.all_plugins = {}
+
         env.menumanager = menu.MenuManager()
-        env.dataprovider = dataprovider.DataProviderManager()
-        env.dataprovider.load()
+        env.dataprovider = dataprovider.DataProviderManager(autoupdate=self.autoupdate)
+        env.dataprovider.load(autoupdate=self.autoupdate)
+        env.linkmanager = link.LinkManager()
 
         env.plugins = {}
-        for i in pluginmanager.PluginManager("prewikka.plugins"):
+        for i in pluginmanager.PluginManager("prewikka.plugins", autoupdate=self.autoupdate):
             try:
-                env.plugins[i.__name__] = i()
-            except error.PrewikkaUserError as err:
-                env.log.warning("%s: plugin loading failed: %s" % (i.__name__, err))
+                env.plugins[i.__name__] = pluginmanager.PluginManager.initialize_plugin(i)
+            except error.PrewikkaUserError:
+                pass
 
         # Load views before auth/session to find all permissions
-        env.viewmanager.loadViews()
+        env.viewmanager.load_views(autoupdate=self.autoupdate)
 
         _AUTH_PLUGINS = pluginmanager.PluginManager("prewikka.auth", autoupdate=True)
         _SESSION_PLUGINS = pluginmanager.PluginManager("prewikka.session", autoupdate=True)
@@ -185,60 +197,57 @@ class Core(object):
         elif cfg.auth:
             # No session module defined, we load the auth module first
             self._load_auth_or_session("auth", _AUTH_PLUGINS, cfg.auth.get_instance_name(), cfg.auth)
-            self._load_auth_or_session("session", _SESSION_PLUGINS, env.auth.getDefaultSession())
+            self._load_auth_or_session("session", _SESSION_PLUGINS, env.auth.get_default_session())
         else:
             # Nothing defined, we use the anonymous module
             self._load_auth_or_session("session", _SESSION_PLUGINS, "anonymous")
             env.auth = env.session
 
-        env.viewmanager.addView(viewhelpers.AjaxHostURL())
-        env.renderer = renderer.RendererPluginManager()
+        env.renderer = renderer.RendererPluginManager(autoupdate=self.autoupdate)
         list(hookmanager.trigger("HOOK_PLUGINS_LOAD"))
 
-
-    def _reload_plugin_if_needed(self):
+    def reload_plugin_if_needed(self):
         if not env.db.has_plugin_changed():
             return
 
         # Some changes happened, and every process has to reload the plugin configuration
         env.log.warning("plugins were activated: triggering reload")
-
-        list(hookmanager.trigger("HOOK_PLUGINS_RELOAD"))
-        hookmanager.unregister()
-        self._loadPlugins()
-
-        env.viewmanager.set_url_adapter(env.request, cache=False)
+        self._unregister_plugin_data()
+        try:
+            self._load_plugins()
+        finally:
+            # Needed for Database object
+            gc.collect()
 
     def _redirect_default(self, request):
-        default_view = env.menumanager.get_default_view()
-        if default_view:
-            url = url_for(default_view.view_endpoint)
+        if env.menumanager.default_endpoint:
+            url = url_for(env.menumanager.default_endpoint)
         else:
             # The configured view does not exist. Fall back to "settings/my_account"
             # which does not require any specific permission.
             url = request.get_baseurl() + "settings/my_account"
 
-        return response.PrewikkaRedirectResponse(url, code=302)
+        return response.PrewikkaRedirectResponse(url)
 
     def _process_static(self, webreq):
         pathkey = webreq.path_elements[0]
+        endpath = webreq.path[len(pathkey) + 2:]
 
         mapping = env.htdocs_mapping.get(pathkey, None)
-        if not mapping:
+        if not (mapping and endpath and endpath.find(".") != -1):
+            # There is no mapping, or no path beyond the mapped portion was provided.
+            # FIXME: .ext check is not clean: proper way to handle this would be to map statics
+            # files to a /static base directory
             return
 
-        path = os.path.abspath(os.path.join(mapping, webreq.path[len(pathkey) + 2:]))
+        path = os.path.abspath(os.path.join(mapping, endpath))
         if not path.startswith(mapping):
-            return response.PrewikkaDirectResponse(code=403, status_text="Request Forbidden")
-
-        # If the path doesn't map to a regular file, let prewikka continue the processing
-        if not os.path.isfile(path):
-            return
+            return response.PrewikkaResponse(code=403, status_text="Request Forbidden")
 
         try:
             return response.PrewikkaFileResponse(path)
-        except Exception as e:
-            return response.PrewikkaDirectResponse(code=404, status_text="File not found")
+        except OSError:
+            return response.PrewikkaResponse(code=404, status_text="File not found")
 
     def _process_dynamic(self, webreq):
         self._prewikka_init_if_needed()
@@ -253,14 +262,15 @@ class Core(object):
         try:
             env.request.user = env.session.get_user(webreq)
             env.request.user.set_locale()
-        except error.PrewikkaError as autherr:
+        except error.PrewikkaError as e:
+            autherr = e
             pass
 
         if webreq.path == "/":
             return self._redirect_default(webreq)
 
         try:
-            view_object = env.viewmanager.loadView(webreq, env.request.user)
+            view_object = env.viewmanager.load_view(webreq, env.request.user)
         except Exception as err:
             raise autherr or err
 
@@ -268,7 +278,11 @@ class Core(object):
             view_object = autherr
 
         resolve.process(env.dns_max_delay)
-        return view_object.respond()
+        ret = view_object.respond()
+        if env.request.user:
+            env.request.user.sync_properties()
+
+        return ret
 
     def process(self, webreq):
         env.request.init(webreq)
@@ -285,4 +299,10 @@ class Core(object):
                 details=err
             ).respond()
 
-        webreq.send_response(response)
+        try:
+            webreq.send_response(response)
+        except socket.error as e:
+            if e.errno != errno.EPIPE:
+                raise
+        finally:
+            env.request.cleanup()
