@@ -1,4 +1,4 @@
-# Copyright (C) 2016-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2016-2019 CS-SI. All Rights Reserved.
 # Author: Antoine Luong <antoine.luong@c-s.fr>
 #
 # This file is part of the Prewikka program.
@@ -20,32 +20,54 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import collections
+import copy
+import itertools
 import os
+import voluptuous
 import yaml
-from copy import copy
 
-from prewikka import error, hookmanager, siteconfig
+from prewikka import error, hookmanager
+from prewikka.utils import cache
+
+
+_SCHEMA = voluptuous.Schema([{
+    "name": str,
+    "icon": str,
+    "default": bool,
+    voluptuous.Required("categories", default=[]): [{
+        "name": str,
+        "icon": str,
+        voluptuous.Required("sections", default=[]): [{
+            voluptuous.Required("name"): str,
+            "icon": str,
+            voluptuous.Required("tabs", default=[]): [str],
+            "default_tab": str,
+            "expand": bool
+        }]
+    }]
+}])
 
 
 class MenuManager(object):
+    default_endpoint = None
+
     """
     Handle section placement in the menus.
     """
     def __init__(self):
-        self._all_sections = set()
         self._declared_sections = {}
         self._loaded_sections = {}
-        self._sorted = False
         self._default_view = None
-        self._default_section = None
-        self._default_tab = None
 
         filename = env.config.interface.get("menu_order", "menu.yml")
         if not os.path.isabs(filename):
-            filename = os.path.join(siteconfig.conf_dir, filename)
+            filename = os.path.join(env.config.basedir, filename)
 
-        with open(filename, "r") as f:
-            self._menus = yaml.load(f)
+        try:
+            with open(filename, "r") as f:
+                self._menus = _SCHEMA(yaml.safe_load(f))
+        except (IOError, yaml.error.YAMLError, voluptuous.Invalid) as e:
+            raise error.PrewikkaUserError(N_("Menu error"), N_("The provided YAML menu is invalid"), details=e)
 
         if not self._menus:
             raise error.PrewikkaUserError(N_("Menu error"), N_("Empty menu"))
@@ -62,25 +84,36 @@ class MenuManager(object):
 
                 default_menu = True
 
-            for category in menu.get("categories", []):
-
-                for section in category.get("sections", []):
-                    if "name" not in section:
-                        raise error.PrewikkaUserError(N_("Menu error"), N_("Section without a name in %s", filename))
-
+            for category in menu["categories"]:
+                for section in category["sections"]:
                     if "default_tab" in section:
-                        if self._default_section:
+                        if self._default_view:
                             raise error.PrewikkaUserError(N_("Menu error"), N_("Multiple default views"))
-                        self._default_section, self._default_tab = (section["name"], section["default_tab"])
 
-                    self._declared_sections[section["name"]] = section.get("tabs", [])
+                        self._default_view = (section["name"], section["default_tab"])
+
+                    self._declared_sections[section["name"]] = collections.OrderedDict((v, idx) for idx, v in enumerate(section["tabs"]))
 
         if not default_menu:
             self._menus[-1]["default"] = True
 
+    @cache.request_memoize("get_sections")
     def get_sections(self):
-        self._sort_tabs()
-        return self._loaded_sections
+        ret = {}
+        _loaded_sections = copy.deepcopy(self._loaded_sections)
+
+        for section, tab, endpoint in itertools.chain.from_iterable(hookmanager.trigger("HOOK_MENU_LOAD")):
+            _loaded_sections.setdefault(section, collections.OrderedDict())[tab] = endpoint
+
+        for section, tabs in _loaded_sections.items():
+            if section not in self._declared_sections:
+                ret[section] = tabs
+            else:
+                ret[section] = collections.OrderedDict()
+                for name in sorted(tabs.keys(), key=lambda tab: self._declared_sections[section].get(tab, 100)):
+                    ret[section][name] = tabs[name]
+
+        return ret
 
     def get_menus(self):
         return self._menus
@@ -88,41 +121,8 @@ class MenuManager(object):
     def get_declared_sections(self):
         return self._declared_sections
 
-    def get_default_view(self):
-        return self._default_view
+    def add_section_info(self, section, tab, endpoint, **kwargs):
+        self._loaded_sections.setdefault(section, collections.OrderedDict())[tab] = (endpoint, kwargs)
 
-    def add_section(self, name):
-        self._all_sections.add(name)
-
-    def add_section_info(self, view):
-        self.add_section(view.view_menu[0])
-
-        self._loaded_sections.setdefault(view.view_menu[0], collections.OrderedDict()) \
-                             .setdefault(view.view_menu[1], collections.OrderedDict())[view.view_id] = view
-
-        if view.view_menu[:2] == (self._default_section, self._default_tab):
-            self._default_view = view
-
-    def _tab_index(self, section, tab):
-        try:
-            return self._declared_sections[section].index(tab)
-        except ValueError:
-            # Arbitrary big number to put the tab at the end
-            return 100
-
-    def _sort_tabs(self):
-        if self._sorted:
-            return
-
-        ret = {}
-
-        for section, views in self._loaded_sections.items():
-            if section not in self._declared_sections:
-                ret[section] = views
-            else:
-                ret[section] = collections.OrderedDict()
-                for name in sorted(views.keys(), key=lambda tab: self._tab_index(section, tab)):
-                    ret[section][name] = views[name]
-
-        self._sorted = True
-        self._loaded_sections = ret
+        if (section, tab) == self._default_view:
+            self.default_endpoint = endpoint
