@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2014-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -19,10 +19,10 @@
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-import json
 import uuid
 
 from prewikka import error, pluginmanager, resource
+from prewikka.utils import cache
 
 RED_STD = "E78D90"
 ORANGE_STD = "F5B365"
@@ -31,16 +31,8 @@ GREEN_STD = "B1E55D"
 BLUE_STD = "93B9DD"
 GRAY_STD = "5C5C5C"
 
-SEVERITY_COLOR_MAP = {
-        "high": (_("High"), RED_STD),
-        "medium": (_("Medium"), ORANGE_STD),
-        "low": (_("Low"), GREEN_STD),
-        "info": (_("Informational"), BLUE_STD)
-}
-
-COLOR_MAP = "93B9DD", "B1E55D", "D4C608", "F5B365", "E78D90", "C6A0CF", "5256D3", \
-            "A7DE65", "F2A97B", "F6818A", "B087C6", "66DC92"
-
+COLORS = (BLUE_STD, GREEN_STD, YELLOW_STD, ORANGE_STD, RED_STD,
+          "C6A0CF", "5256D3", "A7DE65", "F2A97B", "F6818A", "B087C6", "66DC92")
 
 
 class RendererException(Exception):
@@ -52,19 +44,18 @@ class RendererNoDataException(RendererException):
         return _("No data to display.")
 
 
-
 class RendererItem(object):
-        __slots__ = [ "values", "labels", "links", "_tuple" ]
+    __slots__ = ["values", "labels", "links", "_tuple"]
 
-        def __init__(self, values=0, labels=None, links=None):
-                self._tuple = values, labels, links
+    def __init__(self, values=0, labels=None, links=None):
+        self._tuple = values, labels, links
 
-                self.values = values
-                self.labels = labels
-                self.links = links
+        self.values = values
+        self.labels = labels
+        self.links = links
 
-        def __getitem__(self, i):
-                return self._tuple[i]
+    def __getitem__(self, i):
+        return self._tuple[i]
 
 
 class RendererUtils(object):
@@ -72,22 +63,22 @@ class RendererUtils(object):
 
     def __init__(self, options):
         self._color_map_idx = 0
-
-        self._color_map = options.get("names_and_colors", None)
-        if not self._color_map:
-            self._color_map = COLOR_MAP
+        self._color_map = options.get("names_and_colors")
 
     def get_label(self, label):
-        if isinstance(self._color_map, dict):
-            return self._color_map.get(label, self._nexist_color)[0]
+        if self._color_map:
+            return _(self._color_map.get(label, self._nexist_color)[0])
 
         return label
 
+    @cache.request_memoize("renderer_color")
     def get_color(self, label, onecolor=False):
-        if isinstance(self._color_map, dict):
-            return self._color_map.get(label, self._nexist_color)[1]
+        if self._color_map:
+            color = self._color_map.get(label, self._nexist_color)[1]
+            if color:
+                return color
 
-        color = self._color_map[self._color_map_idx % len(self._color_map)]
+        color = COLORS[self._color_map_idx % len(COLORS)]
 
         if not onecolor:
             self._color_map_idx += 1
@@ -95,7 +86,6 @@ class RendererUtils(object):
         return color
 
 
-
 class RendererBackend(pluginmanager.PluginBase):
     pass
 
@@ -103,21 +93,23 @@ class RendererBackend(pluginmanager.PluginBase):
 class RendererPluginManager(pluginmanager.PluginManager):
     _default_backends = {}
 
-    def __init__(self):
-        self._backends = pluginmanager.PluginManager("prewikka.renderer.backend")
-        pluginmanager.PluginManager.__init__(self, "prewikka.renderer.type")
+    def __init__(self, autoupdate=False):
+        self._backends = pluginmanager.PluginManager("prewikka.renderer.backend", autoupdate=autoupdate)
+        pluginmanager.PluginManager.__init__(self, "prewikka.renderer.type", autoupdate=autoupdate)
 
         for typ, backend in env.config.renderer_defaults.items():
-            self._default_backends[typ] = backend.value
+            self._default_backends[typ] = backend
 
         self._renderer = {}
         for i in self:
             try:
-                self._renderer.setdefault(i.renderer_backend, {})[i.renderer_type] = i()
-            except Exception as e:
-                env.log.error("%s: %s" % (i.__module__, e))
+                p = self.initialize_plugin(i)
+            except Exception:
+                continue
 
-            if not i.renderer_type in self._default_backends:
+            self._renderer.setdefault(i.renderer_backend, {})[i.renderer_type] = p
+
+            if i.renderer_type not in self._default_backends:
                 self._default_backends[i.renderer_type] = i.renderer_backend
 
     def get_types(self):
@@ -144,13 +136,12 @@ class RendererPluginManager(pluginmanager.PluginManager):
     def get_default_backend(self, wanted_type):
         return self._default_backends.get(wanted_type)
 
-    def render(self, type, data, renderer=None, **kwargs):
-        if renderer is None:
-            renderer = self.get_default_backend(type)
+    def _setup_renderer(self, type, renderer):
+        renderer = renderer or self.get_default_backend(type)
 
-            if renderer is None:
-                raise error.PrewikkaUserError(N_("Renderer error"),
-                                              N_("No backend supporting render type '%s'", type))
+        if renderer is None:
+            raise error.PrewikkaUserError(N_("Renderer error"),
+                                          N_("No backend supporting render type '%s'", type))
 
         if renderer not in self._renderer:
             raise error.PrewikkaUserError(N_("Renderer error"),
@@ -161,23 +152,26 @@ class RendererPluginManager(pluginmanager.PluginManager):
                                           N_("Backend '%(backend)s' does not support render type '%(type)s'",
                                              {'backend': renderer, 'type': type}))
 
-        if not "names_and_colors" in kwargs:
-            kwargs["names_and_colors"] = COLOR_MAP
+        return renderer
+
+    def update(self, type, data, renderer=None, **kwargs):
+        renderer = self._setup_renderer(type, renderer)
+        return self._renderer[renderer][type].update(data, **kwargs)
+
+    def render(self, type, data, renderer=None, **kwargs):
+        renderer = self._setup_renderer(type, renderer)
 
         classname = kwargs["class"] = "-".join((renderer, type))
         cssid = kwargs["cssid"] = "-".join((classname, text_type(uuid.uuid4())))
 
         try:
             data = self._renderer[renderer][type].render(data, **kwargs)
-            html = resource.HTMLSource("""<div id="%s" class="renderer-elem %s">%s</div>""" % (cssid, classname, data.get("html", "")))
+            htmls = resource.HTMLSource('<div id="%s" class="renderer-elem %s">%s</div>'
+                                        % (cssid, classname, data.get("html", "")))
 
-            return {"html": html, "script": resource.HTMLSource(data.get("script", "")) }
-        except RendererNoDataException as e:
-            html = resource.HTMLSource("""<div id="%s" class="renderer-elem renderer-elem-error %s">%s</div>""" % (cssid, classname, text_type(e)))
-            script = resource.HTMLSource("""
-                 var size = prewikka_getRenderSize("#%s", %s);
+            return {"html": htmls, "script": resource.HTMLSource(data.get("script", ""))}
+        except RendererException as e:
+            htmls = resource.HTMLSource('<div id="%s" class="renderer-elem renderer-elem-error %s"><div class="text-center-vh">%s</div></div>'
+                                        % (cssid, classname, text_type(e)))
 
-                 $("#%s").width(size[0]).css("height", size[1] + 'px').css("line-height", size[1] + 'px');
-                """ % (cssid, json.dumps(kwargs), cssid))
-
-            return {"html": html, "script": script}
+            return {"html": htmls, "script": None}
