@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2008-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -29,7 +29,7 @@ import_fail = None
 
 try:
     from twisted.internet import reactor
-    from twisted.names import client, dns, hosts, cache, resolve
+    from twisted.names import client, dns, cache, resolve
 except Exception as err:
     import_fail = err
 
@@ -45,7 +45,7 @@ class DNSResolver(object):
         self._lock = Lock()
 
         self._cache = cache.CacheResolver()
-        rlist = [ self._cache, client.Resolver('/etc/resolv.conf') ]
+        rlist = [self._cache, client.Resolver('/etc/resolv.conf')]
         self._resolve = resolve.ResolverChain(rlist)
 
     def _error_cb(self, failure):
@@ -87,7 +87,7 @@ class DNSResolver(object):
                 self._lock.release()
                 break
 
-            reactor.runUntilCurrent();
+            reactor.runUntilCurrent()
             reactor.doIteration(timeout)
 
             self._lock.release()
@@ -96,9 +96,9 @@ class DNSResolver(object):
             if end >= final:
                 break
 
-        #print "max=%f elapsed:%f" % (timeout, end-now)
+        # print "max=%f elapsed:%f" % (timeout, end-now)
 
-    def doQuery(self, addr, resolve_cb):
+    def do_query(self, addr, resolve_cb):
         self._lock.acquire()
 
         self._query += 1
@@ -113,7 +113,7 @@ class DNSResolver(object):
         except:
             return
 
-        self.doQuery(addr, resolve_cb)
+        self.do_query(addr, resolve_cb)
         self.process()
 
 
@@ -140,7 +140,7 @@ class AddressResolve(object):
     def __len__(self):
         return len(str(self))
 
-    def resolveSucceed(self):
+    def resolve_succeed(self):
         if self._name:
             return True
         else:
@@ -167,7 +167,7 @@ def init():
         return
 
     if import_fail:
-       env.log.warning(_("Asynchronous DNS resolution disabled: twisted.names and twisted.internet required: %s") % import_fail)
-       return
+        env.log.warning(_("Asynchronous DNS resolution disabled: twisted.names and twisted.internet required: %s") % import_fail)
+        return
 
     resolver = DNSResolver()
