@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2016-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2016-2019 CS-SI. All Rights Reserved.
 # Author: Antoine Luong <antoine.luong@c-s.fr>
 #
 # Inspired by the tw2.core package of ToscaWidgets2 which is Copyright (c)
@@ -52,7 +52,6 @@ class JSLink(Link):
         return Link.__new__(cls, html.Markup('<script type="text/javascript" src="%s"></script>') % link)
 
 
-
 class HTMLSource(html.Markup):
     pass
 
@@ -62,7 +61,7 @@ class CSSSource(HTMLSource):
     An inlined chunk of CSS source code.
     """
     def __new__(cls, src):
-        return HTMLSource.__new__(cls, html.Markup('<style type="text/css">%s</style>') % src)
+        return HTMLSource.__new__(cls, html.Markup('<style type="text/css">%s</style>' % src))
 
 
 class JSSource(HTMLSource):
@@ -70,11 +69,13 @@ class JSSource(HTMLSource):
     An inlined chunk of JS source code.
     """
     def __new__(cls, src):
-        return HTMLSource.__new__(cls, html.Markup('<script type="text/javascript">%s</script>') % src)
+        return HTMLSource.__new__(cls, html.Markup('<script type="text/javascript">%s</script>' % src))
 
 
 @functools.total_ordering
 class HTMLNode(json.JSONObject):
+    _HTML5_VOID_TAGS = frozenset(["area", "base", "br", "col", "embed", "hr", "img", "input", "keygen", "link", "meta", "param", "source", "track", "wbr"])
+
     def __init__(self, tag, *childs, **attrs):
         self.tag = tag
         self.childs = childs
@@ -83,7 +84,8 @@ class HTMLNode(json.JSONObject):
         if icon:
             self.childs = (HTMLNode("i", _class="fa %s" % icon), " ") + self.childs
 
-        self._sortkey = attrs.pop("_sortkey", None)
+        self._sortkey = attrs.pop("_sortkey", "")
+        self._extra = attrs.pop("_extra", None)
 
         tmp = attrs.pop("_class", None)
         if tmp:
@@ -98,7 +100,10 @@ class HTMLNode(json.JSONObject):
                 _class += " %s" % v
                 continue
 
-            attr_s += HTMLSource(" %s=\"%s\"") % (k, v)
+            if v is True:
+                attr_s += HTMLSource(" %s" % k)
+            elif v not in (False, None):
+                attr_s += HTMLSource(" %s=\"%s\"") % (k, v)
 
         if _class:
             attr_s = HTMLSource(" class=\"%s\"") % (_class) + attr_s
@@ -107,14 +112,46 @@ class HTMLNode(json.JSONObject):
         for x in self.childs:
             childs += text_type(x)
 
+        if not self.tag:
+            return HTMLSource("%s" % childs)
+
+        if self.tag in self._HTML5_VOID_TAGS:
+            if self.childs:
+                raise Exception("HTMLNode with void tag '%s' contains children" % self.tag)
+
+            return HTMLSource("<%s%s />" % (self.tag, attr_s))
+
         return HTMLSource("<%s%s>%s</%s>" % (self.tag, attr_s, childs, self.tag))
 
+    def join(self, l):
+        ret = HTMLNode("")
+
+        for i, elem in enumerate(l):
+            if i > 0:
+                ret += self
+
+            ret += elem
+
+        return ret
+
+    def format(self, *args, **kwargs):
+        return self.to_string().format(*args, **kwargs)
+
     def __json__(self):
-        return { "tag": self.tag, "childs": self.childs, "attrs": self.attrs }
+        return {"tag": self.tag, "childs": self.childs, "attrs": self.attrs, "extra": self._extra}
+
+    def __html__(self):
+        return self.to_string()
 
     def __str__(self):
         return self.to_string()
 
+    def __mod__(self, other):
+        return self.to_string() % other
+
+    def __add__(self, other):
+        return HTMLNode(None, self, other)
+
     def __eq__(self, other):
         return (self._sortkey, self.childs) == (other._sortkey, other.childs)
 
