@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 # Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
@@ -26,10 +26,11 @@ import os
 import time
 import datetime
 import dateutil.parser
+import stat
 import string
 import unicodedata
 
-from prewikka import compat, template, utils
+from prewikka import compat, utils
 from prewikka.utils import json
 
 
@@ -63,7 +64,7 @@ class PrewikkaResponse(object):
 
     def __init__(self, data=None, headers=_sentinel, code=None, status_text=None):
         self.data = data
-        self.code = code or 200
+        self.code = code
         self.status_text = status_text
         self.ext_content = {}
 
@@ -85,6 +86,11 @@ class PrewikkaResponse(object):
         """Add an extra content to the response (add in XHR request)."""
 
         self.ext_content[key] = value
+        return self
+
+    def add_html_content(self, elem, target=None):
+        self.ext_content.setdefault("html_content", []).append({"target": target, "html": elem})
+        return self
 
     def add_notification(self, message, classname="success", name=None, icon=None, duration=None):
         """Add notification to the return value."""
@@ -97,46 +103,35 @@ class PrewikkaResponse(object):
             "duration": duration
         })
 
-    @staticmethod
-    def _is_xhr_embedded_content(obj):
-        is_json = hasattr(obj, '__json__')
-        if is_json and not isinstance(obj, template.PrewikkaTemplate):
-            return False
-
-        return True
+        return self
 
     def content(self):
-        """Retrieve the HTML content of the response."""
-        if env.request.web.is_xhr and self._is_xhr_embedded_content(self.data):
-            res = self._with_xhr_layout(self.data)
-        else:
-            res = self.data
+        if self.data is None:
+            if not self.ext_content:
+                return None
 
-        return self._encode_response(res)
+            self.data = {}
+
+        if isinstance(self.data, compat.STRING_TYPES):
+            return self.data
+
+        self.headers["Content-Type"] = "application/json"
+        if isinstance(self.data, dict):
+            self.data["_extensions"] = self.ext_content
+
+        return json.dumps(self.data)
 
     def _encode_response(self, res):
-        if res is None:
-            res = ""
-
-        if not isinstance(res, compat.STRING_TYPES):
-            self.headers["Content-Type"] = "application/json"
-            res = json.dumps(res)
-
         return res.encode(env.config.general.get("encoding", "utf8"), "xmlcharrefreplace")
 
-    def _with_xhr_layout(self, obj):
-        """Position the obj in a dict for XHR response"""
-
-        data = {"type": "html", "content": obj}
-        data.update(self.ext_content)
-
-        return data
-
     def write(self, request):
         content = self.content()
+        if content is None and self.code is None:
+            self.code = 204
 
-        request.send_headers(self.headers.items(), self.code, self.status_text)
-        request.write(content)
+        request.send_headers(self.headers.items(), self.code or 200, self.status_text)
+        if content is not None:
+            request.write(self._encode_response(content))
 
 
 class PrewikkaDownloadResponse(PrewikkaResponse):
@@ -168,7 +163,10 @@ class PrewikkaDownloadResponse(PrewikkaResponse):
         disposition = "inline" if inline else "attachment"
         if filename:
             # As specified in RFC 6266
-            disposition += "; filename=\"%s\"; filename*=utf-8''%s" % (self._filename_to_ascii(filename), utils.url.quote(filename.encode("utf8")))
+            disposition += "; filename=\"%s\"; filename*=utf-8''%s" % (
+                self._filename_to_ascii(filename),
+                utils.url.quote(filename.encode("utf8"))
+            )
 
         self._is_file = not(isinstance(self.data, text_type))
         if not size:
@@ -186,7 +184,7 @@ class PrewikkaDownloadResponse(PrewikkaResponse):
         ))
 
     def write(self, request):
-        request.send_headers(self.headers.items(), self.code, self.status_text)
+        request.send_headers(self.headers.items(), self.code or 200, self.status_text)
 
         if not self._is_file:
             request.write(self.data)
@@ -195,17 +193,6 @@ class PrewikkaDownloadResponse(PrewikkaResponse):
                 request.write(i)
 
 
-class PrewikkaDirectResponse(PrewikkaResponse):
-    """
-        Direct HTML response
-
-        Render the directly the data without wrapping it, even on XHR request.
-    """
-
-    def content(self):
-        return self._encode_response(self.data)
-
-
 class PrewikkaFileResponse(PrewikkaResponse):
     """
         Static File response
@@ -214,25 +201,28 @@ class PrewikkaFileResponse(PrewikkaResponse):
         PrewikkaResponse.__init__(self)
         self._path = path
 
-        stat = os.stat(path)
+        fst = os.stat(path)
+        if not stat.S_ISREG(fst.st_mode):
+            raise Exception("Attempt to send an invalid file")
+
         content_type = mimetypes.guess_type(path)[0] or "application/octet-stream"
-        mtime = datetime.datetime.utcfromtimestamp(stat.st_mtime).replace(tzinfo=utils.timeutil.tzutc())
+        mtime = datetime.datetime.utcfromtimestamp(fst.st_mtime).replace(tzinfo=utils.timeutil.tzutc())
 
         ims = env.request.web.headers.get("if-modified-since")
         if ims is not None:
-            ims = dateutil.parser.parse(ims)
+            ims = dateutil.parser.parse(ims.split(";")[0])  # Edge includes the length in this header
             if mtime <= ims:
                 self.code = 304
 
         self.headers = collections.OrderedDict((('Content-Type', content_type),))
 
         if self.code != 304:
-            self.headers["Content-Length"] = str(stat.st_size)
+            self.headers["Content-Length"] = str(fst.st_size)
             self.headers["Expires"] = (mtime + datetime.timedelta(days=30)).strftime("%a, %d %b %Y %H:%M:%S GMT")
             self.headers["Last-Modified"] = mtime.strftime("%a, %d %b %Y %H:%M:%S GMT")
 
     def write(self, request):
-        request.send_headers(self.headers.items(), self.code, self.status_text)
+        request.send_headers(self.headers.items(), self.code or 200, self.status_text)
         if self.code == 304:
             return
 
@@ -246,5 +236,5 @@ class PrewikkaRedirectResponse(PrewikkaResponse):
         Redirect response
     """
     def __init__(self, location, code=302, status_text=None):
-        PrewikkaResponse.__init__(self, code=code, status_text=status_text or "%d Redirect" % code)
+        PrewikkaResponse.__init__(self, code=code, status_text=status_text)
         self.headers = collections.OrderedDict((('Location', location),))
