@@ -1,2 +1,3 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
-from .anonymous import AnonymousSession
+
+from .anonymous import AnonymousSession  # noqa: imported but unused
