@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -88,12 +88,12 @@ class Session(pluginmanager.PluginBase):
         pluginmanager.PluginBase.__init__(self)
 
         self._db = SessionDatabase()
-        self._expiration = int(config.get("expiration", 60)) * 60
+        self._expiration = config.get_int('expiration', 60) * 60
 
         hookmanager.register("HOOK_USER_DELETE", lambda user: self._db.delete_session(user=user))
 
     def __set_session(self, request, sessionid):
-        request.add_cookie("sessionid", sessionid, self._expiration * 3)
+        request.add_cookie("sessionid", sessionid, expires=self._expiration * 3, httponly=True)
 
     def __check_session(self, request):
         sessionid = request.input_cookie.get("sessionid")
@@ -124,12 +124,14 @@ class Session(pluginmanager.PluginBase):
 
         # And that the user it carry still exist in the current authentication
         # backend (which might have changed)
-        if not(env.auth.hasUser(usergroup.User(login))):
+        user = usergroup.User(login)
+        if not env.auth.has_user(user):
             self.__delete_session(request)
             raise SessionInvalid(login, template=self.template)
 
         if (now - t) / 60 >= 5:
             self._db.update_session(sessionid, now)
+            list(hookmanager.trigger("HOOK_SESSION_UPDATE", user))
 
         self.__set_session(request, sessionid)
         return login
@@ -138,7 +140,7 @@ class Session(pluginmanager.PluginBase):
         t = time.time()
 
         self._db.delete_expired_sessions(t - self._expiration)
-        sessionid = text_type(binascii.hexlify(os.urandom(16) + struct.pack(b">d", t)))
+        sessionid = binascii.hexlify(os.urandom(16) + struct.pack(b">d", t)).decode("utf8")
 
         self._db.create_session(sessionid, user, int(t))
         self.__set_session(request, sessionid)
@@ -160,6 +162,8 @@ class Session(pluginmanager.PluginBase):
         user = self.authenticate(request, info)
         self.__create_session(request, user)
 
+        list(hookmanager.trigger("HOOK_SESSION_CREATE", user))
+
         is_admin = set(user.permissions) == usergroup.ALL_PERMISSIONS
         env.log.info("User login with profile '%s'" % ("admin" if is_admin else "default"))
 
@@ -171,6 +175,7 @@ class Session(pluginmanager.PluginBase):
     def logout(self, request):
         login = self.__check_session(request)
         self.__delete_session(request)
+        list(hookmanager.trigger("HOOK_SESSION_DELETE", usergroup.User(login)))
         env.log.info("Logged out")
 
         raise SessionInvalid(message=N_("Logged out"),
