@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2014-2019 CS-SI. All Rights Reserved.
 # Author: Antoine Luong <antoine.luong@c-s.fr>
 #
 # This file is part of the Prewikka program.
@@ -26,7 +26,7 @@ import os
 import pkg_resources
 
 
-def getThemes():
+def get_themes():
     theme_list = []
 
     css_files = glob.glob(pkg_resources.resource_filename(__name__, "htdocs/css/themes/*.css"))
