@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -20,15 +20,20 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import abc
+import copy
 import hashlib
 
-from prewikka import compat, error, localization, log, utils
+from prewikka import compat, error, hookmanager, localization, log, utils
 from prewikka.utils import cache, json
 
 ADMIN_LOGIN = "admin"
 _NAMEID_TBL = {}
 
 
+class UnknownNameIDError(KeyError):
+    pass
+
+
 class PermissionDeniedError(error.PrewikkaUserError):
     def __init__(self, permissions, view=None):
         if isinstance(permissions, compat.STRING_TYPES):
@@ -71,6 +76,7 @@ class Permissions(set):
 
 
 ALL_PERMISSIONS = Permissions()
+ACTIVE_PERMISSIONS = Permissions()
 
 
 class NameID(object):
@@ -123,50 +129,56 @@ class NameID(object):
     def __str__(self):
         return self.name
 
+    def __repr__(self):
+        try:
+            name = self.name
+        except UnknownNameIDError:
+            return "%s(id=%s)" % (self.__class__.__name__, self.id)
+
+        return "%s(name=%s, id=%s)" % (self.__class__.__name__, name, self.id)
+
 
 class Group(NameID):
     def __init__(self, name=None, groupid=None):
         NameID.__init__(self, name, groupid)
 
     def _id2name(self, id):
-        return env.auth.getGroupByID(id).name
+        group = env.auth.get_group_by_id(id)
+        if not group:
+            raise UnknownNameIDError(id)
 
+        return group.name
 
-def _sync_if_needed(func):
-    def inner(self, *args, **kwargs):
-        if self._properties_state:
-            return func(self, *args, **kwargs)
-
-        self.begin_properties_change()
-        ret = func(self, *args, **kwargs)
-        self.commit_properties_change()
-
-        return ret
-
-    return inner
+    def create(self):
+        env.auth.create_group(self)
+        list(hookmanager.trigger("HOOK_GROUP_CREATE", self))
 
+    def delete(self):
+        list(hookmanager.trigger("HOOK_GROUP_DELETE", self))
+        env.auth.delete_group(self)
 
 
 class User(NameID):
     __sentinel = object()
-    __PROPERTIES_STATE_NONE  = 0x00
-    __PROPERTIES_STATE_BEGIN = 0x01
-    __PROPERTIES_STATE_DIRTY = 0x02
 
     def __init__(self, login=None, userid=None):
+        self._orig_configuration = None
         NameID.__init__(self, login, userid)
-        self._properties_state = self.__PROPERTIES_STATE_NONE
 
     def _id2name(self, id):
-        return env.auth.getUserByID(id).name
+        user = env.auth.get_user_by_id(id)
+        if not user:
+            raise UnknownNameIDError(id)
+
+        return user.name
 
     @cache.request_memoize_property("user_permissions")
     def permissions(self):
-        return set(env.auth.getUserPermissions(self))
+        return set(env.auth.get_user_permissions(self))
 
     @permissions.setter
     def permissions(self, permissions):
-        env.auth.setUserPermissions(self, permissions)
+        env.auth.set_user_permissions(self, permissions)
 
     def _permissions(self, permissions):
         self.permissions  # make sure the cache has been created
@@ -175,13 +187,25 @@ class User(NameID):
     # Support access to _permissions to modify object permission without backend modification.
     _permissions = property(permissions, _permissions)
 
-    @cache.request_memoize_property("user_configuration")
+    @property
     def configuration(self):
+        ret = self._configuration()
+        if self._orig_configuration is None:
+            self._orig_configuration = copy.deepcopy(ret)
+
+        return ret
+
+    @cache.request_memoize("user_configuration")
+    def _configuration(self):
         rows = env.db.query("SELECT config FROM Prewikka_User_Configuration WHERE userid = %s", self.id)
         if rows:
             return json.loads(rows[0][0])
+        else:
+            return {}
 
-        return {}
+    @configuration.setter
+    def configuration(self, conf):
+        env.request.cache.user_configuration._set((self,), conf)
 
     @cache.request_memoize_property("user_timezone")
     def timezone(self):
@@ -190,28 +214,18 @@ class User(NameID):
     def set_locale(self):
         lang = self.get_property("language", default=env.config.general.default_locale)
         if lang:
-            localization.setLocale(lang)
+            localization.set_locale(lang)
 
-    @_sync_if_needed
     def del_property(self, key, view=None):
         view = view or ""
 
         if not key:
-            r = self.configuration.pop(view, None)
+            self.configuration.pop(view, None)
         else:
-            r = self.configuration.get(view, {}).pop(key, None)
+            self.configuration.get(view, {}).pop(key, None)
 
-        if r:
-            self._properties_state |= self.__PROPERTIES_STATE_DIRTY
-
-    def delete(self):
-        env.db.query("DELETE FROM Prewikka_User_Configuration WHERE userid = %s", self.id)
-
-    @_sync_if_needed
     def del_properties(self, view):
-        r = self.configuration.pop(view, None)
-        if r:
-            self._properties_state |= self.__PROPERTIES_STATE_DIRTY
+        self.configuration.pop(view, None)
 
     def del_property_match(self, key, view=None):
         view = view or ""
@@ -239,22 +253,16 @@ class User(NameID):
     def get_property(self, key, view=None, default=None):
         return self.get_property_fail(key, view or "", default)
 
-    @_sync_if_needed
     def set_property(self, key, value, view=None):
-        old = self.configuration.get(view or "", {}).get(key)
-        if old != value:
-            self._properties_state |= self.__PROPERTIES_STATE_DIRTY
-
         self.configuration.setdefault(view or "", {})[key] = value
 
-    def begin_properties_change(self):
-        self._properties_state = self.__PROPERTIES_STATE_BEGIN
-
-    def commit_properties_change(self):
-        if self._properties_state & self.__PROPERTIES_STATE_DIRTY:
+    def sync_properties(self):
+        if self._orig_configuration is not None and self._orig_configuration != self.configuration:
+            self._orig_configuration = copy.deepcopy(self.configuration)
             env.db.upsert("Prewikka_User_Configuration", ["userid", "config"], [[self.id, json.dumps(self.configuration)]], pkey=("userid",))
 
-        self._properties_state = self.__PROPERTIES_STATE_NONE
+        if env.request.user == self:
+            env.request.user = self
 
     def has(self, perm):
         if type(perm) in (list, tuple, set):
@@ -265,3 +273,12 @@ class User(NameID):
     def check(self, perm, view=None):
         if not self.has(perm):
             raise PermissionDeniedError(perm, view)
+
+    def create(self):
+        env.auth.create_user(self)
+        list(hookmanager.trigger("HOOK_USER_CREATE", self))
+
+    def delete(self):
+        list(hookmanager.trigger("HOOK_USER_DELETE", self))
+        env.db.query("DELETE FROM Prewikka_User_Configuration WHERE userid = %s", self.id)
+        env.auth.delete_user(self)
