@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 # Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
@@ -20,29 +20,28 @@
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+import os
 import os.path
 import re
 import struct
 import sys
-import time
 import unicodedata
 
-from prewikka import compat
-from prewikka.utils import json
+from prewikka.utils.timeutil import now
 
 
 port_dict = {}
 read_done = False
 
-if sys.version_info >= (3,0):
+if sys.version_info >= (3, 0):
     text_type = str
 else:
     text_type = unicode
 
 
 class AttrObj(object):
-    def __init__(self, **kwargs):
-        self.__dict__.update(kwargs)
+    def __init__(self, *args, **kwargs):
+        self.__dict__.update(*args, **kwargs)
 
     def __json__(self):
         return self.__dict__
@@ -53,6 +52,18 @@ class AttrObj(object):
     def __eq__(self, other):
         return self.__dict__ == other.__dict__
 
+    def __iter__(self):
+        if sys.version_info >= (3, 0):
+            return self.__dict__.items()
+        else:
+            return self.__dict__.iteritems()
+
+    def __getitem__(self, val):
+        return self.__dict__.__getitem__(val)
+
+    def keys(self):
+        return self.__dict__.keys()
+
 
 # FIXME: Need appropriate implementation
 def get_analyzer_status_from_latest_heartbeat(heartbeat, error_margin):
@@ -63,7 +74,7 @@ def get_analyzer_status_from_latest_heartbeat(heartbeat, error_margin):
     if heartbeat.get("heartbeat_interval") is None:
         return "unknown", _("Unknown")
 
-    if time.time() - int(heartbeat.get("create_time")) > int(heartbeat.get("heartbeat_interval")) + error_margin:
+    if (now() - heartbeat["create_time"]).total_seconds() > heartbeat["heartbeat_interval"] + error_margin:
         return "missing", _("Missing")
 
     return "online", _("Online")
@@ -79,7 +90,8 @@ def _load_protocol():
     read_done = True
     sreg = re.compile("^\s*(?P<name>[^#]\S+)\s*(?P<number>\d+)\s*(?P<alias>\S+)")
 
-    try: fd = open("/etc/protocols", "r")
+    try:
+        fd = open("/etc/protocols", "r")
     except IOError:
         return port_dict
 
@@ -96,26 +108,17 @@ def _load_protocol():
 
 
 def protocol_number_to_name(num):
-     port_dict = _load_protocol()
+    port_dict = _load_protocol()
 
-     if num in port_dict:
-         return port_dict[num][0]
+    if num in port_dict:
+        return port_dict[num][0]
 
-     return None
+    return None
 
 
-def nameToPath(name):
-    if not isinstance(name, compat.STRING_TYPES):
-        name = text_type(name)
-
-    return name.lower().replace(" ", "_")
-
-
-def find_unescaped_characters(value, characters=None):
+def find_unescaped_characters(value, characters):
     """Search for unescaped characters among *characters* in string *value*."""
     escaped = False
-    if not characters:
-        return False
 
     for char in value:
         if escaped:
@@ -157,12 +160,14 @@ def soundex(name):
     sndx = ''
     fc = ''
 
-    name = unicodedata.normalize("NFKD", name).encode("ascii", "ignore")
+    # We need to call text_type() so that we work on a string (not bytes) with Py3
+    name = text_type(unicodedata.normalize("NFKD", name).encode("ascii", "ignore"))
 
     # translate alpha chars in name to soundex digits
     for i, c in enumerate(name):
         if c.isalpha():
-            if not fc: fc = c   # remember first letter
+            if not fc:
+                fc = c   # remember first letter
 
             idx = ord(c.upper()) - ord('A')
             if idx >= len(digits):
@@ -221,6 +226,25 @@ def deprecated(func):
     return new_func
 
 
+def path_sort_key(path):
+    """
+    Return a key to sort dataprovider paths in natural order,
+    so that alert.source(10) comes after alert.source(2).
+    """
+    return [int(part) if part.isdigit() else part for part in re.split("(\d+)", path)]
+
+
+def get_file_size(fileobj):
+    """
+    Return the size (in bytes) of a file-like object (like StringIO).
+    """
+    position = fileobj.tell()
+    fileobj.seek(0, os.SEEK_END)
+    size = fileobj.tell()
+    fileobj.seek(position)
+    return size
+
+
 class CachingIterator(object):
     def __init__(self, items, count=None):
         self._count = count
@@ -253,7 +277,7 @@ class CachingIterator(object):
         self._items = iter([])
 
     def __getitem__(self, key):
-        if isinstance(key, slice) :
+        if isinstance(key, slice):
             return [self[i] for i in range(*key.indices(len(self)))]
 
         elif isinstance(key, int):
@@ -267,3 +291,6 @@ class CachingIterator(object):
             raise IndexError
 
         return self._cache[key]
+
+    def __json__(self):
+        return list(self)
