@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -23,17 +23,16 @@ import base64
 import errno
 import os.path
 import sys
-import prelude
 import random
 
 from prewikka import siteconfig
 
 
-if sys.version_info >= (3,0):
-    from urllib.parse import quote, unquote, urlsplit, urlunsplit, urlencode as _urlencode
+if sys.version_info >= (3, 0):
+    from urllib.parse import quote, quote_plus, urlparse, urlsplit, urlunsplit, urlencode as _urlencode  # noqa: imported but unused
 else:
-    from urllib import quote, unquote, urlencode as __urlencode
-    from urlparse import urlsplit, urlunsplit
+    from urllib import quote, quote_plus, urlencode as __urlencode  # noqa: imported but unused
+    from urlparse import urlparse, urlsplit, urlunsplit  # noqa: imported but unused
 
     def _convert(d):
         if isinstance(d, (list, tuple)):
@@ -66,7 +65,7 @@ class mkdownload(object):
     def __init__(self, filename, mode="wb+", user=True, inline=False):
         self.name = filename
         self._id = random.randint(0, 9999999)
-        self._dlname = base64.urlsafe_b64encode(filename.encode("utf8"))
+        self._dlname = base64.urlsafe_b64encode(filename.encode("utf8")).decode("utf8")
         filename = self.get_filename(self._id, self._dlname, user)
 
         try:
@@ -89,7 +88,7 @@ class mkdownload(object):
     def get_filename(cls, id, filename, user=True):
         user = cls._get_user(user)
         if user:
-            user = base64.urlsafe_b64encode(user.encode("utf8"))
+            user = base64.urlsafe_b64encode(user.encode("utf8")).decode("utf8")
 
         return os.path.join(cls.DOWNLOAD_DIRECTORY, user or "", "%d-%s" % (id, filename))
 
@@ -111,7 +110,7 @@ class mkdownload(object):
         return getattr(self.fd, attr)
 
     def __json__(self):
-        return { "type": "download", "href": self.href }
+        return {"type": "download", "href": self.href}
 
 
 def iri2uri(iri, encoding="utf8"):
@@ -125,11 +124,11 @@ def iri2uri(iri, encoding="utf8"):
     else:
         authority = tpl[0].encode('idna') + ":%s" % tpl[1]
 
-    return urlunsplit((scheme.encode(encoding), authority,
+    return urlunsplit((scheme, text_type(authority, encoding),
                        quote(path.encode(encoding), safe),
                        quote(query.encode(encoding), safe),
                        quote(frag.encode(encoding), safe)))
 
 
 def urlencode(parameters, doseq=False):
-    return _urlencode(parameters, doseq).replace('&', '&amp;')
+    return _urlencode(parameters, doseq)
