@@ -1,4 +1,4 @@
-# Copyright (C) 2016-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2016-2019 CS-SI. All Rights Reserved.
 # Author: Antoine Luong <antoine.luong@c-s.fr>
 #
 # This file is part of the Prewikka program.
@@ -19,9 +19,15 @@
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-from string import Template
+from prewikka import response, utils, view
 
-from prewikka import hookmanager, resource, response, utils, view
+
+def GridParameters(name):
+    class _GridParameters(view.Parameters):
+        def register(self):
+            self.optional("jqgrid_params_%s" % name, utils.json.loads, {}, persist=True)
+
+    return _GridParameters
 
 
 class GridAjaxParameters(view.Parameters):
@@ -40,41 +46,12 @@ class GridAjaxParameters(view.Parameters):
         return {"total": nb_pages, "page": self["page"], "rows": [], "records": total_results}
 
 
-class GridAjaxResponse(response.PrewikkaDirectResponse):
-    def __init__(self, rows, total_results):
-        response.PrewikkaDirectResponse.__init__(self)
+class GridAjaxResponse(response.PrewikkaResponse):
+    def __init__(self, rows, total_results, **kwargs):
+        response.PrewikkaResponse.__init__(self)
 
         # Ceil division (use // instead of / for Python3 compatibility):
-        nb_pages = (total_results - 1) // int(env.request.parameters.get("rows", 10)) + 1
-        self.data = {"total": nb_pages, "rows": rows, "records": total_results}
-
-
-class AjaxHostURL(view.View):
-    class AjaxHostURLParameters(view.Parameters):
-        def register(self):
-            self.mandatory("host", text_type)
-
-    view_parameters = AjaxHostURLParameters
-
-    @staticmethod
-    def _value_generator(infos):
-        for urlname, url in env.url.get("host", {}).items():
-            try:
-                url = Template(url).substitute(infos)
-            except KeyError:
-                continue
-
-            yield _(urlname), url
-
-    @classmethod
-    def _link_generator(cls, infos):
-        for urlname, url in cls._value_generator(infos):
-            yield resource.Link('<a href="{url}" target="{urlname}">{urlname}</a>').format(urlname=urlname, url=url)
-
-    def render(self):
-        infos = {"host": env.request.parameters["host"]}
-
-        for info in hookmanager.trigger("HOOK_HOST_INFO", env.request.parameters["host"]):
-            infos.update(info)
-
-        return response.PrewikkaDirectResponse(list(self._link_generator(infos)))
+        kwargs["total"] = (total_results - 1) // int(env.request.parameters.get("rows", 10)) + 1
+        kwargs["rows"] = rows
+        kwargs["records"] = total_results
+        self.data = kwargs
