@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2014-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -23,7 +23,7 @@ import itertools
 import json
 import pkg_resources
 
-from prewikka import database, error, pluginmanager, template, utils, version, view, response
+from prewikka import cli, database, error, response, template, utils, version, view
 from prewikka.utils import html
 
 
@@ -37,16 +37,16 @@ class AboutPlugin(view.View):
     plugin_mandatory = True
     plugin_htdocs = (("aboutplugin", pkg_resources.resource_filename(__name__, 'htdocs')),)
 
-    view_permissions = [ N_("USER_MANAGEMENT") ]
+    view_permissions = [N_("USER_MANAGEMENT")]
 
-    _all_plugins = ((_("Apps: View"), "prewikka.views"),
-                    (_("Apps: API"), "prewikka.plugins"),
-                    (_("Apps: Dataprovider backend"), "prewikka.dataprovider.backend"),
-                    (_("Apps: Dataprovider type"), "prewikka.dataprovider.type"),
-                    (_("Apps: Authentication"), "prewikka.auth"),
-                    (_("Apps: Identification"), "prewikka.session"),
-                    (_("Apps: Renderer backend"), "prewikka.renderer.backend"),
-                    (_("Apps: Renderer type"), "prewikka.renderer.type"))
+    _all_plugins = ((N_("Apps: View"), "prewikka.views"),
+                    (N_("Apps: API"), "prewikka.plugins"),
+                    (N_("Apps: Dataprovider backend"), "prewikka.dataprovider.backend"),
+                    (N_("Apps: Dataprovider type"), "prewikka.dataprovider.type"),
+                    (N_("Apps: Authentication"), "prewikka.auth"),
+                    (N_("Apps: Identification"), "prewikka.session"),
+                    (N_("Apps: Renderer backend"), "prewikka.renderer.backend"),
+                    (N_("Apps: Renderer type"), "prewikka.renderer.type"))
 
     def _add_plugin_info(self, data, catname, mod):
         dbup = database.DatabaseUpdateHelper(mod.full_module_name, mod.plugin_database_version, mod.plugin_database_branch)
@@ -58,14 +58,14 @@ class AboutPlugin(view.View):
                 data.maintenance_total += len(upinfo)
                 data.maintenance.setdefault(catname, []).append((mod, curversion, upinfo))
             else:
-                data.installed.setdefault(catname, []).append((mod, env.db.is_plugin_active(mod.full_module_name)))
+                data.installed.setdefault(catname, []).append((mod, env.db.is_plugin_active(mod)))
 
         except error.PrewikkaUserError as e:
             data.maintenance.setdefault(catname, []).append((mod, curversion, [e]))
 
     def _iter_plugin(self):
         for catname, entrypoint in self._all_plugins:
-            for plugin in pluginmanager.PluginManager.iter_plugins(entrypoint):
+            for plugin in env.all_plugins[entrypoint].values():
                 yield catname, plugin
 
     def _get_plugin_infos(self):
@@ -79,6 +79,15 @@ class AboutPlugin(view.View):
 
         return data
 
+    @cli.register("list", "plugin", help=N_("list plugin: list installed plugins"))
+    def _list_plugins(self):
+        ret = []
+        for plugins in self._get_plugin_infos().installed.values():
+            for mod, active in plugins:
+                ret.append(mod.full_module_name)
+
+        return sorted(ret)
+
     @view.route("/settings/apps", methods=["GET"], menu=(N_("Apps"), N_("Apps")), help="#apps")
     def render_get(self):
         dset = template.PrewikkaTemplate(__name__, "templates/aboutplugin.mak").dataset()
@@ -96,19 +105,26 @@ class AboutPlugin(view.View):
 
         for catname, plugin in self._iter_plugin():
             enabled = plugin.plugin_mandatory or plugin.full_module_name in env.request.parameters["enable_plugin"]
-            upsrt.append((plugin.full_module_name, int(enabled)))
+            upsrt.append((plugin.full_module_name, enabled))
 
         if upsrt:
             env.db.upsert("Prewikka_Module_Registry", ["module", "enabled"], upsrt, pkey=["module"])
             env.db.trigger_plugin_change()
 
-        return response.PrewikkaDirectResponse({"type": "reload"})
+        return response.PrewikkaResponse({"type": "reload", "target": "window"})
 
     @view.route("/settings/apps/update", methods=["GET"])
     def update(self):
+        self._update(env.request.web.send_stream)
+
+    @cli.register("sync", "plugin", help=N_("sync plugin: initialize the plugin database schemas"))
+    def _update_plugins(self):
+        self._update(lambda *args, **kwargs: None)
+
+    def _update(self, send_stream):
         data = self._get_plugin_infos()
 
-        env.request.web.send_stream(json.dumps({"total": data.maintenance_total}), event="begin", sync=True)
+        send_stream(json.dumps({"total": data.maintenance_total}), event="begin", sync=True)
 
         for mod, fromversion, uplist in itertools.chain.from_iterable(data.maintenance.values()):
             for upscript in uplist:
@@ -116,16 +132,16 @@ class AboutPlugin(view.View):
                     continue
 
                 label = _("Applying %(module)s %(script)s...") % {'module': mod.full_module_name, 'script': text_type(upscript)}
-                env.request.web.send_stream(json.dumps({"label": html.escape(label), 'module': html.escape(mod.full_module_name), 'script': html.escape(text_type(upscript))}), sync=True)
+                send_stream(json.dumps({"label": html.escape(label), 'module': html.escape(mod.full_module_name), 'script': html.escape(text_type(upscript))}), sync=True)
 
                 try:
                     upscript.apply()
                 except Exception as e:
-                    env.request.web.send_stream(json.dumps({"logs": "\n".join(html.escape(x) for x in upscript.query_logs), "error": html.escape(text_type(e))}), sync=True)
+                    send_stream(json.dumps({"logs": "\n".join(html.escape(x) for x in upscript.query_logs), "error": html.escape(text_type(e))}), sync=True)
                 else:
-                    env.request.web.send_stream(json.dumps({"logs": "\n".join(html.escape(x) for x in upscript.query_logs), "success": True}), sync=True)
+                    send_stream(json.dumps({"logs": "\n".join(html.escape(x) for x in upscript.query_logs), "success": True}), sync=True)
 
-        env.request.web.send_stream(data=json.dumps({"label": _("All updates applied")}), event="finish", sync=True)
-        env.request.web.send_stream("close", event="close")
+        send_stream(data=json.dumps({"label": _("All updates applied")}), event="finish", sync=True)
+        send_stream("close", event="close")
 
         env.db.trigger_plugin_change()
