@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -20,14 +20,14 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import hashlib
+import itertools
 import re
 import string
 import struct
 import urllib
-from datetime import datetime
 
 import pkg_resources
-from prewikka import hookmanager, localization, resolve, resource, template, utils, view, mainmenu
+from prewikka import hookmanager, localization, resolve, resource, template, utils, view
 from prewikka.dataprovider import Criterion
 from prewikka.utils import html
 
@@ -35,9 +35,9 @@ from prewikka.utils import html
 def getUriCriteria(ptype, analyzerid, messageid):
     criteria = Criterion()
     if analyzerid:
-        criteria += Criterion("%s.analyzer.analyzerid" % (ptype), "=", analyzerid)
+        criteria += Criterion("%s.analyzer.analyzerid" % ptype, "=", analyzerid)
 
-    return criteria + Criterion("%s.messageid" % (ptype), "=", messageid)
+    return criteria + Criterion("%s.messageid" % ptype, "=", messageid)
 
 
 class Table(object):
@@ -49,10 +49,10 @@ class Table(object):
         return self._current_section
 
     def beginSection(self, title, display="block"):
-        _current_section = { }
+        _current_section = {}
         _current_section["title"] = title
-        _current_section["entries"] = [ ]
-        _current_section["tables"] = [ ]
+        _current_section["entries"] = []
+        _current_section["tables"] = []
         _current_section["display"] = display
         _current_section["sections"] = []
         _current_section["parent"] = self._current_section
@@ -79,9 +79,9 @@ class Table(object):
         if value is None or value == "":
             return
 
-        self._current_section["entries"].append({ "name": name,
-                                                  "value": value,
-                                                  "emphase": emphase })
+        self._current_section["entries"].append({"name": name,
+                                                 "value": value,
+                                                 "emphase": emphase})
 
     def beginTable(self, cl="table-striped table-bordered", style="", odd_even=False):
         table = {}
@@ -92,13 +92,11 @@ class Table(object):
         table["parent"] = self._current_table or self._current_section
         self._current_table = table
 
-
     def endTable(self):
         parent = self._current_table["parent"]
-        has_data = False
 
-        if len(self._current_table["rows"]) <= 1 :
-            if not parent or not "rows" in parent:
+        if len(self._current_table["rows"]) <= 1:
+            if not parent or "rows" not in parent:
                 self._current_table = None
             else:
                 self._current_table = parent
@@ -109,7 +107,7 @@ class Table(object):
             self._current_table = None
         else:
             if "rows" in parent:
-                col = { "name": None, "header": None, "emphase": None, "tables": [ self._current_table ] }
+                col = {"name": None, "header": None, "emphase": None, "tables": [self._current_table]}
                 if len(parent["rows"]):
                     parent["rows"][-1] += [col]
                 else:
@@ -126,7 +124,7 @@ class Table(object):
         return len(self._current_table["rows"])
 
     def newTableCol(self, row_index, name, cl="", header=False, emphase=None):
-        col = { "name": name, "header": header, "class": cl, "tables": [], "emphase": None }
+        col = {"name": name, "header": header, "class": cl, "tables": [], "emphase": None}
 
         if row_index == -1:
             self._current_table["rows"].append([col])
@@ -138,17 +136,16 @@ class Table(object):
             self._current_table["rows"][row_index] += [col]
 
     def newTableEntry(self, name, value, cl="", emphase=False):
-        if value == None:
+        if value is None:
             return
 
         self.newTableCol(0, name, cl=cl, header=True)
         self.newTableCol(1, value, cl=cl, header=False, emphase=emphase)
 
 
-
 class HeaderTable(Table):
     def __init__(self):
-        self.field_list = [ ]
+        self.field_list = []
 
     def register_static(self, name, static):
         self.field_list.append((None, name, static, None, None))
@@ -223,7 +220,6 @@ class TcpIpOptions(Table):
         return md.hexdigest()
 
     def _decodeOptionPartialOrderProfile(self, data):
-        x = struct.unpack(b">B", data)
         return "Start_Flags=%d End_Flags=%d" % (data & 0x80, data & 0x40)
 
     def _decodeOptionTcpAltChecksumRequest(self, data):
@@ -240,7 +236,6 @@ class TcpIpOptions(Table):
         else:
             return "%d (Invalid)" % x
 
-
     def _tcpOptionToName(self, opt):
         h = {}
         h[0] = ("End of Option List", 0)
@@ -273,7 +268,6 @@ class TcpIpOptions(Table):
 
         return h.get(opt, ("Unknown",))
 
-
     def _ipOptionToName(self, opt):
         h = {}
         h[0] = ("End of Option List", 0)
@@ -324,7 +318,6 @@ class TcpIpOptions(Table):
 
         self._optionRender(ip_options, self._ipOptionToName)
 
-
     def tcpOptionRender(self, tcp_options):
         if not tcp_options:
             return
@@ -335,20 +328,9 @@ class TcpIpOptions(Table):
         self._optionRender(tcp_options, self._tcpOptionToName)
 
 
-
-class MessageParameters(mainmenu.MainMenuParameters):
-    def register(self):
-        mainmenu.MainMenuParameters.register(self)
-        view.Parameters.register(self)
-        self.optional("analyzerid", text_type)
-        self.optional("messageid", text_type)
-
-
 class MessageSummary(Table, view.View):
-    view_parameters = MessageParameters
-    view_permissions = [ N_("IDMEF_VIEW") ]
+    view_permissions = [N_("IDMEF_VIEW")]
     view_template = template.PrewikkaTemplate(__name__, 'templates/messagesummary.mak')
-    view_extensions = (("menu", mainmenu.MainMenu),)
     plugin_htdocs = (("messagesummary", pkg_resources.resource_filename(__name__, 'htdocs')),)
 
     def __init__(self, *args, **kwargs):
@@ -375,8 +357,8 @@ class MessageSummary(Table, view.View):
         if not dt:
             return None
 
-        agent_time = datetime.fromtimestamp(dt, utils.timeutil.tzoffset(None, dt.getGmtOffset()))
-        user_time = datetime.fromtimestamp(dt, env.request.user.timezone)
+        agent_time = dt
+        user_time = dt.astimezone(env.request.user.timezone)
 
         s = localization.format_datetime(user_time, format="medium")
         if agent_time.tzinfo.utcoffset(agent_time) != user_time.tzinfo.utcoffset(user_time):
@@ -406,7 +388,6 @@ class MessageSummary(Table, view.View):
         self.newTableEntry(_("Process PID"), process["pid"])
         self.endTable()
 
-
     def buildNode(self, node):
         if not node:
             return
@@ -430,7 +411,7 @@ class MessageSummary(Table, view.View):
         if node["name"]:
             self.newTableEntry(_("Node name"), node["name"])
 
-        elif node_name is not None and node_name.resolveSucceed():
+        elif node_name is not None and node_name.resolve_succeed():
             self.newTableEntry(_("Node name (resolved)"), node_name)
 
         if addr_list:
@@ -454,7 +435,7 @@ class MessageSummary(Table, view.View):
 
         self.buildNode(analyzer["node"])
         if analyzer["ostype"] or analyzer["osversion"]:
-                self.newTableEntry(_("Operating System"), "%s %s" % (analyzer["ostype"] or "", analyzer["osversion"] or ""))
+            self.newTableEntry(_("Operating System"), "%s %s" % (analyzer["ostype"] or "", analyzer["osversion"] or ""))
 
         self.endTable()
         self.newTableRow()
@@ -465,9 +446,9 @@ class MessageSummary(Table, view.View):
 
         self.endTable()
 
-    def buildAnalyzerList(self, alert):
+    def buildAnalyzerList(self, msg):
         l = []
-        for analyzer in alert["analyzer"]:
+        for analyzer in msg["analyzer"]:
             l.insert(0, analyzer)
 
         l.pop(0)
@@ -488,7 +469,7 @@ class MessageSummary(Table, view.View):
         self.endTable()
         self.endSection()
 
-    def buildAdditionalData(self, alert, ignore=[], ignored={}, ip_options=[], tcp_options=[]):
+    def buildAdditionalData(self, msg, ptype, ignore=[], ignored={}, ip_options=[], tcp_options=[]):
         self.beginSection(_("Additional data"))
 
         self.beginTable()
@@ -496,7 +477,7 @@ class MessageSummary(Table, view.View):
         self.newTableCol(0, _("Value"), header=True)
 
         index = 1
-        for ad in alert["additional_data"]:
+        for ad in msg["additional_data"]:
             value = None
             meaning = ad["meaning"]
 
@@ -518,26 +499,27 @@ class MessageSummary(Table, view.View):
                 tcp_options[-1] = (tcp_options[-1][0], len(data), data)
                 ignored[meaning] = ""
 
-            if ad["data"] != None:
+            if ad["data"] is not None:
                 value = ad["data"]
                 if ad["type"] == "byte-string" and meaning != "payload":
-                    value = utils.hexdump(value)
+                    value = html.escape(utils.hexdump(value)).replace(" ", resource.HTMLSource("&nbsp;"))
+                    value = resource.HTMLSource("<span class='fixed'>%s</span>" % value)
 
             for field in ignore:
-                if meaning != None and meaning == field[0]:
+                if meaning is not None and meaning == field[0]:
                     ignored[meaning] = value
                     break
 
             links = resource.HTMLSource()
-            for obj in filter(None, hookmanager.trigger("HOOK_ALERTSUMMARY_MEANING_LINK", alert, meaning, value)):
-                links += obj.to_string()
+            for obj in filter(None, hookmanager.trigger("HOOK_%sSUMMARY_MEANING_LINK" % ptype.upper(), msg, meaning, value)):
+                links += obj
 
             if links:
                 meaning = resource.HTMLSource("<a class='popup_menu_toggle'>%s</a><span class='popup_menu'>%s</span>") % (meaning, links)
 
-            if not meaning in ignored:
+            if meaning not in ignored:
                 self.newTableCol(index, resource.HTMLSource(meaning or "Data content"))
-                self.newTableCol(index, html.escape(value) if value else None)
+                self.newTableCol(index, html.escape(value) if value is not None else None)
                 index += 1
 
         self.endTable()
@@ -597,7 +579,7 @@ class MessageSummary(Table, view.View):
         icmp.register(_("Checksum"), "icmp_sum")
         icmp.register(_("Id"), "icmp_id")
         icmp.register(_("Seq #"), "icmp_seq")
-        icmp.register(_("Mask"), "icmp_mask");
+        icmp.register(_("Mask"), "icmp_mask")
         icmp.register(_("Gateway Address"), "icmp_gwaddr")
         icmp.register(_("Num address"), "icmp_num_addrs")
         icmp.register(_("Wpa"), "icmp_wpa")
@@ -611,7 +593,7 @@ class MessageSummary(Table, view.View):
     def buildPayloadTable(self, alert):
         data = HeaderTable()
         data.register(_("Payload"), "payload")
-        #data.register("ASCII Payload", "payload", html.escape)
+        # data.register("ASCII Payload", "payload", html.escape)
         return data
 
     def render(self):
@@ -628,7 +610,7 @@ class AlertSummary(TcpIpOptions, MessageSummary):
         return Criterion("alert.analyzer.analyzerid", "=", analyzerid) & Criterion("alert.messageid", "=", ident)
 
     def buildAlertIdent(self, alert, parent):
-        calist = { }
+        calist = {}
 
         for alertident in parent["alertident"]:
 
@@ -645,26 +627,38 @@ class AlertSummary(TcpIpOptions, MessageSummary):
 
             calist.setdefault(analyzerid, []).append(alertident["alertident"])
 
-        for idx, analyzerid in enumerate(calist):
-
+        for idx, (analyzerid, idents) in enumerate(calist.items()):
             content = ""
-            criteria = Criterion()
-            for ident in calist[analyzerid]:
-                criteria |= self._get_alert_ident_criterion(analyzerid, ident)
+            results = []
+            total = 0
+            step = 50
+            limit = min(len(idents), 500)
 
-            results = env.dataprovider.query(["alert.messageid", "alert.classification.text"], criteria)
+            for i in range(0, limit, step):
+                # FIXME #3250, #3251
+                # We execute several queries to avoid recursion errors
+                criteria = Criterion()
+                for ident in idents[i:i+step]:
+                    criteria |= self._get_alert_ident_criterion(analyzerid, ident)
 
-            for ident, classif in results:
-                link = url_for(".", analyzerid=analyzerid, messageid=ident)
+                results.append(env.dataprovider.query(["alert.messageid", "alert.classification.text"], criteria))
+
+            for ident, classif in itertools.chain(*results):
+                link = url_for(".render", analyzerid=analyzerid, messageid=ident)
                 content += '<li><a title="%s" href="%s">%s</a></li>' % (_("Alert details"), link, html.escape(classif))
+                total += 1
 
-            missing = len(calist[analyzerid]) - len(results)
+            missing = limit - total
             if missing > 0:
                 content += "<li>" + (_("%d linked alerts missing (probably deleted)") % missing) + "</li>"
 
+            omitted = len(idents) - limit
+            if omitted > 0:
+                content += "<li>" + (_("%d linked alerts omitted") % omitted) + "</li>"
+
             self.newTableCol(idx + 1, resource.HTMLSource("<ul style='padding: 0px; margin: 0px 0px 0px 10px;'>%s</ul>" % content))
 
-            linked_alerts = env.dataprovider.get(self._get_alert_ident_criterion(analyzerid, ident))
+            linked_alerts = env.dataprovider.get(self._get_alert_ident_criterion(analyzerid, idents[0]))
             if linked_alerts:
                 self.buildAnalyzer(linked_alerts[0]["alert.analyzer(-1)"])
             else:
@@ -728,10 +722,11 @@ class AlertSummary(TcpIpOptions, MessageSummary):
 
             if env.enable_details:
                 if reference["origin"] in ("user-specific", "vendor-specific"):
-                    urlstr="&url=" + urllib.quote(reference["url"], safe="")
+                    urlstr = "&url=" + urllib.quote(reference["url"], safe="")
                 else:
-                    urlstr=""
-                self.newTableCol(index, self.getUrlLink(reference["name"], "%s?origin=%s&name=%s%s" % (env.reference_details_url, urllib.quote(reference["origin"]), urllib.quote(reference["name"]), urlstr)))
+                    urlstr = ""
+                url = "%s?origin=%s&name=%s%s" % (env.reference_details_url, urllib.quote(reference["origin"]), urllib.quote(reference["name"]), urlstr)
+                self.newTableCol(index, self.getUrlLink(reference["name"], url))
             else:
                 self.newTableCol(index, reference["name"])
             self.newTableCol(index, reference["meaning"])
@@ -763,7 +758,7 @@ class AlertSummary(TcpIpOptions, MessageSummary):
 
     def _joinUserInfos(self, user, number, tty=None):
         user_str = user or ""
-        if user != None and number != None:
+        if user is not None and number is not None:
             user_str += "(%d)" % number
 
         elif number:
@@ -786,7 +781,7 @@ class AlertSummary(TcpIpOptions, MessageSummary):
 
         index = 1
         for user_id in user["user_id"]:
-            #user_str = self._joinUserInfos(user_id["name"], user_id["number"], user_id["tty"])
+            # user_str = self._joinUserInfos(user_id["name"], user_id["number"], user_id["tty"])
             self.newTableCol(index, user_id["type"])
             self.newTableCol(index, user_id["name"])
             self.newTableCol(index, user_id["number"])
@@ -892,7 +887,6 @@ class AlertSummary(TcpIpOptions, MessageSummary):
         self.endTable()
         self.endSection()
 
-
     def buildService(self, service):
         if not service:
             return
@@ -928,11 +922,11 @@ class AlertSummary(TcpIpOptions, MessageSummary):
             self.newTableEntry(_("ip_version"), service["ip_version"])
 
         ipn = service["iana_protocol_number"]
-        if ipn and utils.protocol_number_to_name(ipn) != None:
+        if ipn and utils.protocol_number_to_name(ipn) is not None:
             self.newTableEntry(_("Protocol"), utils.protocol_number_to_name(ipn))
 
         elif service["iana_protocol_name"]:
-             self.newTableEntry(_("Protocol"), service["iana_protocol_name"])
+            self.newTableEntry(_("Protocol"), service["iana_protocol_name"])
 
         elif service["protocol"]:
             self.newTableEntry(_("Protocol"), service["protocol"])
@@ -1014,7 +1008,7 @@ class AlertSummary(TcpIpOptions, MessageSummary):
 
         alert = env.dataprovider.get(getUriCriteria("alert", analyzerid, messageid))[0]["alert"]
 
-        env.request.dataset["sections"] = [ ]
+        env.request.dataset["sections"] = []
 
         self.beginSection(self.getSectionName(alert))
 
@@ -1059,7 +1053,7 @@ class AlertSummary(TcpIpOptions, MessageSummary):
         tcp_options = []
 
         group = ip.field_list + tcp.field_list + udp.field_list + icmp.field_list + data.field_list
-        self.buildAdditionalData(alert, ignore=group, ignored=ignored_value, ip_options=ip_options, tcp_options=tcp_options)
+        self.buildAdditionalData(alert, "alert", ignore=group, ignored=ignored_value, ip_options=ip_options, tcp_options=tcp_options)
 
         if len(ignored_value.keys()) > 0:
             def blah(b):
@@ -1104,7 +1098,7 @@ class HeartbeatSummary(MessageSummary):
 
         heartbeat = env.dataprovider.get(getUriCriteria("heartbeat", analyzerid, messageid))[0]["heartbeat"]
 
-        env.request.dataset["sections"] = [ ]
+        env.request.dataset["sections"] = []
 
         self.beginSection(_("Heartbeat"))
         self.buildTime(heartbeat)
@@ -1117,4 +1111,4 @@ class HeartbeatSummary(MessageSummary):
         self.endSection()
         self.endSection()
 
-        self.buildAdditionalData(heartbeat)
+        self.buildAdditionalData(heartbeat, "heartbeat")
