@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 # Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
@@ -21,10 +21,10 @@
 from __future__ import absolute_import, division, print_function
 
 import abc
+import cgi
 import sys
 
 from prewikka import error
-from prewikka.response import PrewikkaDirectResponse, PrewikkaResponse
 
 if sys.version_info >= (3, 0):
     from http import cookies
@@ -33,23 +33,23 @@ else:
 
 
 class BufferedWriter(object):
-        def __init__(self, wcb, buffersize=8192):
-                self._wcb = wcb
-                self._dlist = []
-                self._len = 0
-                self._buffersize = buffersize
+    def __init__(self, wcb, buffersize=8192):
+        self._wcb = wcb
+        self._dlist = []
+        self._len = 0
+        self._buffersize = buffersize
 
-        def flush(self):
-                self._wcb(''.join(self._dlist))
-                self._dlist = []
-                self._len = 0
+    def flush(self):
+        self._wcb(b''.join(self._dlist))
+        self._dlist = []
+        self._len = 0
 
-        def write(self, data):
-                self._dlist.append(data)
-                self._len += len(data)
+    def write(self, data):
+        self._dlist.append(data)
+        self._len += len(data)
 
-                if self._len >= self._buffersize:
-                        self.flush()
+        if self._len >= self._buffersize:
+            self.flush()
 
 
 class Request(object):
@@ -67,7 +67,18 @@ class Request(object):
         cookie = cookies.SimpleCookie(self.get_cookie())
         self.input_cookie = dict(cookie.items())
 
-    def add_cookie(self, param, value, expires, path="/"):
+    def _handle_multipart(self, **kwargs):
+        arguments = []
+        fs = cgi.FieldStorage(**kwargs)
+
+        for key in fs.keys():
+            value = fs[key]
+            for i, f in enumerate(value if isinstance(value, list) else [value]):
+                arguments.append((key, f if f.filename else f.value))
+
+        return arguments
+
+    def add_cookie(self, param, value, expires=None, path="/", httponly=False):
         if not self._output_cookie:
             self._output_cookie = cookies.SimpleCookie()
 
@@ -76,10 +87,16 @@ class Request(object):
             value = value.encode("utf8")
 
         self._output_cookie[param] = value
-        self._output_cookie[param]["expires"] = expires
+
+        if expires:
+            self._output_cookie[param]["expires"] = expires
+
         if path:
             self._output_cookie[param]["path"] = path
 
+        if httponly:
+            self._output_cookie[param]["httpOnly"] = httponly
+
     def delete_cookie(self, param):
         self.add_cookie(param, "deleted", 0)
 
@@ -92,20 +109,20 @@ class Request(object):
             self.send_headers([("Content-Type", "text/event-stream")])
 
             if retry:
-                self._buffer.write("retry: %d\n" % retry)
+                self._buffer.write(b"retry: %d\n" % retry)
 
         # Join is used in place of concatenation / formatting, because we
         # prefer performance over readability in this place
         if event:
-            self._buffer.write("".join(["event: ", event.encode("utf8"), "\n"]))
+            self._buffer.write(b"".join([b"event: ", event.encode("utf8"), b"\n"]))
 
         if data:
-            self._buffer.write("".join(["data: ", data.encode("utf8"), "\n\n"]))
+            self._buffer.write(b"".join([b"data: ", data.encode("utf8"), b"\n\n"]))
 
         if sync:
             self._buffer.flush()
 
-    def send_response(self, response, code=200, status_text=None):
+    def send_response(self, response):
         """Send a PrewikkaResponse response."""
 
         if self.is_stream:
