@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2017 CS-SI. All Rights Reserved.
+# Copyright (C) 2015-2019 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -20,23 +20,26 @@
 from __future__ import absolute_import, division, print_function
 
 import sys
-import urllib
+import werkzeug.wsgi
 import wsgiref.headers
 import wsgiref.util
 
-from prewikka import main
+from prewikka import main, utils
 from prewikka.web import request
 
+from prewikka.compat.jquery_unparam import jquery_unparam
+
+
 if sys.version_info >= (3, 0):
     Py3 = True
-    import urllib.parse
 else:
     Py3 = False
-    import urlparse
 
 
 defined_status = {
     200: 'Ok',
+    302: 'Found',
+    303: 'See Other',
     304: 'Not Modified',
     400: 'Bad Request',
     401: 'Unauthorized',
@@ -55,14 +58,18 @@ class WSGIRequest(request.Request):
         # Under Python 3, non-ASCII values in the WSGI environ are arbitrarily
         # decoded with ISO-8859-1. This is wrong for Prewikka where UTF-8 is the
         # default. Re-encode to recover the original bytestring.
-        return value.encode("ISO-8859-1") if Py3 else value
+        if value is not None:
+            return value.encode("ISO-8859-1") if Py3 else value
 
     def _wsgi_get_unicode(self, key, default=None):
-        return self._wsgi_get_bytes(key, default).decode("utf8")
+        value = self._wsgi_get_bytes(key, default)
+        if value is not None:
+            return value.decode("utf8")
 
     def _wsgi_get_str(self, key, default=None):
         value = self._wsgi_get_bytes(key, default)
-        return value.decode("utf8") if Py3 else value
+        if value is not None:
+            return value.decode("utf8") if Py3 else value
 
     def __init__(self, environ, start_response):
         self._write = None
@@ -72,22 +79,17 @@ class WSGIRequest(request.Request):
         self.method = environ['REQUEST_METHOD']
 
         request.Request.__init__(self, self._wsgi_get_unicode("PATH_INFO"))
+        self.arguments = jquery_unparam(self._wsgi_get_str("QUERY_STRING"))
 
-        if self.method != 'POST':
-            qs = self._wsgi_get_str("QUERY_STRING")
-        else:
-            qs = environ['wsgi.input'].read(int(environ['CONTENT_LENGTH']))
-            qs = self.body = qs.decode("utf8") if Py3 else qs
+        if self.method in ('POST', 'PUT', 'PATCH'):
+            multipart = environ.get('CONTENT_TYPE', "").startswith('multipart/form-data')
+            if multipart:
+                arg = self._handle_multipart(fp=environ['wsgi.input'], environ=environ)
+            else:
+                arg = environ['wsgi.input'].read(int(environ['CONTENT_LENGTH']))
+                self.body = arg = arg.decode("utf8") if Py3 else arg
 
-        if Py3:
-            self.arguments = urllib.parse.parse_qs(qs)
-        else:
-            self.arguments = {}
-            for k, v in urlparse.parse_qsl(qs):
-                self.arguments.setdefault(k.decode("utf8"), []).append(v.decode("utf8"))
-
-        for name, value in self.arguments.items():
-            self.arguments[name] = (len(value) == 1) and value[0] or value
+            self.arguments.update(jquery_unparam(arg, multipart=multipart))
 
         if self._environ.get("HTTP_X_REQUESTED_WITH", "") == "XMLHttpRequest":
             self.is_xhr = True
@@ -96,6 +98,21 @@ class WSGIRequest(request.Request):
         if self._environ.get("HTTP_ACCEPT", "text/html") == "text/event-stream":
             self.is_stream = True
 
+    def get_target_origin(self):
+        return "%s://%s" % (self._environ.get("wsgi.url_scheme"), werkzeug.wsgi.get_host(self._environ))
+
+    def get_origin(self):
+        ret = self._wsgi_get_unicode("HTTP_ORIGIN")
+        if ret:
+            return ret
+
+        ret = self._wsgi_get_unicode("HTTP_REFERER")
+        if not ret:
+            return None
+
+        scheme, netloc, path, query, frag = utils.url.urlsplit(ret)
+        return "%s://%s" % (scheme, netloc)
+
     def write(self, data):
         self._write(data)
 
@@ -103,11 +120,11 @@ class WSGIRequest(request.Request):
     def headers_sent(self):
         return bool(self._write)
 
-    def send_headers(self, headers=None, code=200, status_text=None):
-        if sys.version_info[0] >= 3:
-            headers = list(headers)
-        else:
-            headers = [ (k.encode("ISO-8859-1"), v.encode("ISO-8859-1")) for k, v in headers ]
+    def send_headers(self, headers=[], code=200, status_text=None):
+        headers = list(headers) + [("X-responseURL", utils.iri2uri(self.get_uri()))]
+
+        if sys.version_info[0] < 3:
+            headers = [(k.encode("ISO-8859-1"), v.encode("ISO-8859-1")) for k, v in headers]
 
         if self._output_cookie:
             headers += [("Set-Cookie", c.OutputString()) for c in self._output_cookie.values()]
@@ -157,7 +174,7 @@ class WSGIRequest(request.Request):
 
 
 def application(environ, start_response):
-        core = main.Core.from_config(environ.get("PREWIKKA_CONFIG", None))
-        core.process(WSGIRequest(environ, start_response))
+    core = main.Core.from_config(environ.get("PREWIKKA_CONFIG", None))
+    core.process(WSGIRequest(environ, start_response))
 
-        return []
+    return []
